@@ -138,6 +138,7 @@ struct glflags_s {
     Dwarf_Bool gf_macro_flag; /* DWARF5 */
     Dwarf_Bool gf_pubnames_flag;
     Dwarf_Bool gf_debug_addr_flag;
+    Dwarf_Bool gf_print_language_version_table;
     Dwarf_Bool gf_ranges_flag; /* .debug_ranges section. */
     Dwarf_Bool gf_reloc_flag;  /* Elf relocations, not DWARF. */
     Dwarf_Bool gf_static_func_flag;/* SGI only */
@@ -150,6 +151,7 @@ struct glflags_s {
     Dwarf_Bool gf_print_utf8_flag;
     Dwarf_Bool gf_print_section_allocations;
     Dwarf_Bool gf_allocation_via_mmap;
+    Dwarf_Bool gf_print_all_srcfiles;
 
     Dwarf_Bool gf_header_flag; /* Control printing of Elf header. */
     Dwarf_Bool gf_section_groups_flag;
