@@ -51,10 +51,14 @@
 #include <config.h>
 
 #include <stdlib.h> /* calloc() free() */
+#include <stdio.h> /* FILE decl for dd_esb.h */
 
 #include "dwarf.h"
 #include "libdwarf.h"
 #include "libdwarf_private.h"
+#include "dd_defined_types.h"
+#include "dd_checkutil.h"
+#include "dd_glflags.h"
 #include "dd_globals.h"
 #include "dd_tsearchbal.h"
 #include "dd_macrocheck.h"
@@ -81,23 +85,25 @@ void *  macro_check_tree;   /* DWARF5 macros */
 void *  macinfo_check_tree; /* DWARF 2,3,4 macros */
 void *  macdefundeftree;    /* DWARF5 macros */
 
-static struct Macrocheck_Map_Entry_s * macrocheck_map_insert(
+static struct Macrocheck_Map_Entry_s * macrocheck_map_insert
+(
     Dwarf_Unsigned off,
-    unsigned prim,unsigned sec,
+    Dwarf_Unsigned prim,
+    Dwarf_Unsigned sec,
     Dwarf_Unsigned linenum,
     unsigned src_file_num,
     void **map);
 static void macrocheck_map_destroy(void *map);
 static Dwarf_Unsigned macro_count_recs(void **base);
 
-#ifdef SELFTEST
+#ifdef TESTING
 int failcount = 0;
 struct glflags_s glflags;
-#endif /* SELFTEST */
+#endif /* TESTING */
 
 static struct Macrocheck_Map_Entry_s *
 macrocheck_map_create_entry(Dwarf_Unsigned offset,
-    unsigned add_primary, unsigned add_secondary)
+    Dwarf_Unsigned add_primary, Dwarf_Unsigned add_secondary)
 {
     struct Macrocheck_Map_Entry_s *mp =
         (struct Macrocheck_Map_Entry_s *)
@@ -135,14 +141,14 @@ macrocheck_map_compare_func(const void *l, const void *r)
 
 static struct Macrocheck_Map_Entry_s *
 macrocheck_map_insert(Dwarf_Unsigned offset,
-    unsigned add_prim,unsigned add_sec,
+    Dwarf_Unsigned add_prim,Dwarf_Unsigned add_sec,
     Dwarf_Unsigned line_num,
     unsigned src_file_num,
     void **tree1)
 {
     void *retval = 0;
     struct Macrocheck_Map_Entry_s *re = 0;
-    struct Macrocheck_Map_Entry_s *e;
+    struct Macrocheck_Map_Entry_s *e = 0;
 
     e  = macrocheck_map_create_entry(offset,add_prim,add_sec);
     /*  tsearch records e's contents unless e
@@ -195,14 +201,6 @@ macrocheck_map_destroy(void *map)
     dwarf_tdestroy(map,macrocheck_map_free_func);
 }
 
-void
-add_macro_import_sup(void **base UNUSEDARG,
-    Dwarf_Unsigned offset UNUSEDARG)
-{
-    /* FIXME */
-    return;
-}
-
 void
 add_macro_import(void **base,
     Dwarf_Bool is_primary,
@@ -237,10 +235,12 @@ add_macro_area_len(void **base, Dwarf_Unsigned offset,
 static Dwarf_Unsigned reccount = 0;
 
 static void
-macro_walk_count_recs(const void *nodep UNUSEDARG,
+macro_walk_count_recs(const void *nodep,
     const DW_VISIT which,
-    const int depth UNUSEDARG)
+    const int depth)
 {
+    (void)nodep;
+    (void)depth;
     if (which == dwarf_postorder || which == dwarf_endorder) {
         return;
     }
@@ -258,11 +258,12 @@ static Dwarf_Unsigned lowestoff = 0xffffff;
 static Dwarf_Bool lowestfound = FALSE;
 static void
 macro_walk_find_lowest(const void *nodep,const DW_VISIT  which,
-    const int  depth UNUSEDARG)
+    const int  depth)
 {
     struct Macrocheck_Map_Entry_s * re =
         *(struct Macrocheck_Map_Entry_s**)nodep;
 
+    (void)depth;
     if (which == dwarf_postorder || which == dwarf_endorder) {
         return;
     }
@@ -308,11 +309,12 @@ static struct Macrocheck_Map_Entry_s **mac_as_array = 0;
 static unsigned mac_as_array_next = 0;
 static void
 macro_walk_to_array(const void *nodep,const DW_VISIT  which,
-    const int  depth UNUSEDARG)
+    const int  depth)
 {
     struct Macrocheck_Map_Entry_s * re =
         *(struct Macrocheck_Map_Entry_s**)nodep;
 
+    (void)depth;
     if (which == dwarf_postorder || which == dwarf_endorder) {
         return;
     }
@@ -345,7 +347,7 @@ qsort_compare(const void *lin, const void *rin)
 static void
 warnprimeandsecond(struct Macrocheck_Map_Entry_s *r)
 {
-#ifdef SELFTEST
+#ifdef TESTING
         ++failcount;
 #endif
     glflags.gf_count_major_errors++;
@@ -370,11 +372,10 @@ warnprimeandsecond(struct Macrocheck_Map_Entry_s *r)
 int
 print_macrocheck_statistics(const char *name,void **tsbase,
     int isdwarf5,
-    Dwarf_Unsigned section_size,
-    Dwarf_Error * err UNUSEDARG)
+    Dwarf_Unsigned section_size)
 {
     Dwarf_Unsigned count = 0;
-    Dwarf_Unsigned lowest = -1ll;
+    Dwarf_Unsigned lowest = (Dwarf_Unsigned)-1ll;
     Dwarf_Unsigned highest = 0;
     Dwarf_Unsigned lastend = 0;
     Dwarf_Unsigned laststart = 0;
@@ -396,7 +397,7 @@ print_macrocheck_statistics(const char *name,void **tsbase,
     mac_as_array = (struct Macrocheck_Map_Entry_s **)calloc(count,
         sizeof(struct Macrocheck_Map_Entry_s *));
     if (!mac_as_array) {
-#ifdef SELFTEST
+#ifdef TESTING
         ++failcount;
 #endif
         glflags.gf_count_major_errors++;
@@ -414,16 +415,6 @@ print_macrocheck_statistics(const char *name,void **tsbase,
         qsort_compare);
     for (i = 0; i < count ; ++i) {
         struct Macrocheck_Map_Entry_s *r = mac_as_array[i];
-#if 0
-        printf("debugging: i %u off 0x%x len 0x%x printed? %u "
-            " ref prim: %u  sec: %u\n",
-            (unsigned)i,
-            (unsigned)r->mp_key,
-            (unsigned)r->mp_len,
-            (unsigned)r->mp_printed,
-            (unsigned)r->mp_refcount_primary,
-            (unsigned)r->mp_refcount_secondary);
-#endif
         if (r->mp_key < lowest) {
             lowest = r->mp_key;
         }
@@ -432,7 +423,7 @@ print_macrocheck_statistics(const char *name,void **tsbase,
             highest = end;
         }
         if (r->mp_refcount_primary > 1) {
-#ifdef SELFTEST
+#ifdef TESTING
             ++failcount;
 #endif
             glflags.gf_count_major_errors++;
@@ -463,17 +454,11 @@ print_macrocheck_statistics(const char *name,void **tsbase,
         lowest, highest);
     for (i = 1; i < count ; ++i) {
         struct Macrocheck_Map_Entry_s *r = mac_as_array[i];
-#if 0
-        printf("debugging i %u off 0x%x len 0x%x\n",
-            (unsigned)i,
-            (unsigned)r->mp_key,
-            (unsigned)r->mp_len);
-#endif
         if (r->mp_key > lastend) {
             internalgap += (r->mp_key - lastend);
         } else if (r->mp_key < lastend) {
             /* crazy overlap */
-#ifdef SELFTEST
+#ifdef TESTING
             ++failcount;
 #endif
             glflags.gf_count_major_errors++;
@@ -502,7 +487,7 @@ print_macrocheck_statistics(const char *name,void **tsbase,
     wholegap = mac_as_array[0]->mp_key + internalgap;
     if (lastend > section_size) {
         /* Something seriously wrong */
-#ifdef SELFTEST
+#ifdef TESTING
         ++failcount;
 #endif
         printf(" ERROR: For offset 0x%" DW_PR_XZEROS DW_PR_DUx
@@ -622,92 +607,3 @@ macro_import_stack_cleanout(void)
 {
     macro_import_stack_next_to_use = 0;
 }
-
-#ifdef SELFTEST
-int
-main()
-{
-    void * base = 0;
-    Dwarf_Unsigned count = 0;
-    int basefailcount = 0;
-    Dwarf_Error err = 0;
-    int isdwarf5=FALSE;
-
-    /* Test 1 */
-    add_macro_import(&base,TRUE,200,0,0);
-    count = macro_count_recs(&base);
-    if (count != 1) {
-        printf("FAIL: expect count 1, got %" DW_PR_DUu "\n",count);
-        ++failcount;
-    }
-    print_macrocheck_statistics("test1",&base,isdwarf5,2000,&err);
-
-    /* Test two */
-    add_macro_area_len(&base,200,100);
-    add_macro_import(&base,FALSE,350,0,0);
-    add_macro_area_len(&base,350,100);
-    count = macro_count_recs(&base);
-    if (count != 2) {
-        printf("FAIL: expect count 2, got %" DW_PR_DUu "\n",count);
-        ++failcount;
-    }
-    print_macrocheck_statistics("test 2",&base,isdwarf5,2000,&err);
-    clear_macrocheck_statistics(&base);
-
-    /* Test three */
-    basefailcount = failcount;
-    add_macro_import(&base,TRUE,0,0,0);
-    add_macro_area_len(&base,0,1000);
-    add_macro_import(&base,FALSE,2000,0,0);
-    add_macro_area_len(&base,2000,100);
-    mark_macro_offset_printed(&base,2000);
-    add_macro_import(&base,FALSE,1000,0,0);
-    add_macro_area_len(&base,1000,900);
-    add_macro_import(&base,FALSE,1000,0,0);
-    add_macro_area_len(&base,1000,900);
-    count = macro_count_recs(&base);
-    if (count != 3) {
-        printf("FAIL: expect count 3, got %" DW_PR_DUu "\n",count);
-        ++failcount;
-    }
-    printf("\n  Expect an ERROR about overlap with "
-        "the end of section\n");
-    print_macrocheck_statistics("test 3",&base,isdwarf5,2000,&err);
-    clear_macrocheck_statistics(&base);
-    if ((basefailcount+1) != failcount) {
-        printf("FAIL: Found no error in test 3 checking!\n");
-        ++failcount;
-    } else {
-        failcount = basefailcount;
-    }
-
-    /* Test Four */
-    basefailcount = failcount;
-    add_macro_import(&base,TRUE,50,0,0);
-    add_macro_import(&base,TRUE,50,0,0);
-    add_macro_area_len(&base,50,50);
-    add_macro_import(&base,FALSE,200,0,0);
-    add_macro_import(&base,FALSE,50,0,0);
-    add_macro_import(&base,FALSE,60,0,0);
-    add_macro_area_len(&base,60,10);
-    printf( "\n  Expect an ERROR about offset 50 having "
-        "2 primaries\n");
-    printf( "  and Expect an ERROR about offset 50 having 2\n"
-        "  primaries"
-        " and a secondary\n");
-    printf( "  and Expect an ERROR about crazy overlap 60\n\n");
-    print_macrocheck_statistics("test 4",&base,isdwarf5,2000,&err);
-    clear_macrocheck_statistics(&base);
-    if ((basefailcount + 3) != failcount) {
-        printf("FAIL: Found wrong errors in test 4 checking!\n");
-    } else {
-        failcount = basefailcount;
-    }
-    if (failcount > 0) {
-        printf("FAIL macrocheck selftest\n");
-        exit(1);
-    }
-    printf("PASS macrocheck selftest\n");
-    return 0;
-}
-#endif /* SELFTEST */
