@@ -62,6 +62,7 @@ Portions Copyright 2012 SN Systems Ltd. All rights reserved.
 #include "locale.h"
 #include "langinfo.h"
 #endif /* HAVE_UTF8 */
+#include <string.h>
 
 #include "dwarf.h"
 #include "libdwarf.h"
@@ -91,6 +92,7 @@ Portions Copyright 2012 SN Systems Ltd. All rights reserved.
 #include "dd_mac_cputype.h"
 #include "dd_elf_cputype.h"
 #include "dd_pe_cputype.h"
+#include "dd_all_srcfiles.h"
 
 #ifndef O_RDONLY
 /*  This is for a Windows environment */
@@ -543,20 +545,21 @@ main(int argc, char *argv[])
 
                 esb_constructor(&m);
                 errmsg = dwarf_errmsg_by_number(errcode);
-                homeify((char *)sanitized(tied_file_name),&m);
+                homeify((char *)tied_file_name,&m);
                 printf("%s ERROR:  can't open tied file"
                     ".. %s: %s\n",
                     glflags.program_name,
-                    esb_get_string(&m),
+                    sanitized(esb_get_string(&m)),
                     errmsg);
                 esb_destructor(&m);
             } else {
                 struct esb_s m;
                 esb_constructor(&m);
-                homeify((char *)sanitized(tied_file_name),&m);
+                homeify((char *)tied_file_name,&m);
                 printf(
                     "%s ERROR: tied file not an object file '%s'.\n",
-                    glflags.program_name, esb_get_string(&m));
+                    glflags.program_name,
+                    sanitized(esb_get_string(&m)));
                 esb_destructor(&m);
             }
             glflags.gf_count_major_errors++;
@@ -577,8 +580,8 @@ main(int argc, char *argv[])
                 "main file \'%s\' not "
                 "the same kind of object!\n",
                 glflags.program_name,
-                esb_get_string(&m),
-                esb_get_string(&mgf));
+                sanitized(esb_get_string(&m)),
+                sanitized(esb_get_string(&mgf)));
             esb_destructor(&m);
             esb_destructor(&mgf);
             free(temp_path_buf);
@@ -1021,15 +1024,7 @@ calculate_likely_limits_of_code(Dwarf_Debug dbg,
         return DW_DLV_NO_ENTRY;
     }
 
-    if (dw_ftype != DW_FTYPE_ELF) {
-        lncount = LIKELYNAMESMAX;
-        memset(likely_names,0,sizeof(likely_names));
-        res = limit_of_code_non_elf(dbg,
-            likely_names,
-            lncount,
-            &basesize,&baselow);
-        ln = likely_names;
-    } else {
+    if (dw_ftype == DW_FTYPE_ELF ) {
         lncount = (int)dwarf_get_section_count(dbg);
         if (!lncount) {
             return DW_DLV_NO_ENTRY;
@@ -1052,6 +1047,14 @@ calculate_likely_limits_of_code(Dwarf_Debug dbg,
             ln = 0;
             return res;
         }
+    } else {
+        lncount = LIKELYNAMESMAX;
+        memset(likely_names,0,sizeof(likely_names));
+        res = limit_of_code_non_elf(dbg,
+            likely_names,
+            lncount,
+            &basesize,&baselow);
+        ln = likely_names;
     }
 
     qsort(ln,lncount,sizeof(struct likely_names_s),
@@ -1093,42 +1096,6 @@ homeify(char *s, struct esb_s* out)
     char *home = getenv("HOME");
     size_t homelen = 0;
 
-#ifdef _WIN32
-    /*  Windows In msys2
-        $HOME might be C:\msys64\home\admin
-        which messes up regression testing.
-        For msys2 with a simple setup this
-        helps regressiontesting.
-    */
-    char *winprefix = "C:/msys64/home/";
-    char *domain = getenv("USERDOMAIN");
-    char *user = getenv("USER");
-    size_t winlen = 15;
-
-    if (domain && !strcmp(domain,"MSYS")) {
-
-        if (strncmp(s,winprefix,winlen)) {
-            /* giving up, not msys2 */
-            esb_append(out,s);
-            return;
-        }
-        if (user) {
-            /*  \\home\\admin
-                Change to $HOME
-                This is a crude way to get some
-                regressiontests to pass.
-            */
-            size_t userlen = strlen(user);
-            esb_append(out,"$HOME");
-            esb_append(out,s+winlen+userlen);
-            return;
-        } else {
-            /* giving up */
-            esb_append(out,s);
-            return;
-        }
-    }
-#endif /* _WIN32 */
     if (!home) {
         /* giving up */
         esb_append(out,s);
@@ -1153,7 +1120,7 @@ homeify(char *s, struct esb_s* out)
         return;
     }
     esb_append(out,"$HOME");
-    /*  Append, starting at the / in x */
+    /*  Append, starting at the / in s */
     esb_append(out,s+homelen);
     return;
 }
@@ -1194,7 +1161,8 @@ process_one_file(
     {
         /*  This will go for the real main file, whether
             an underlying dSYM or via debuglink or
-            if those find nothing then the original. */
+            if those find nothing then the original.
+            Unless glflags.gf_no_follow_debuglink is set! */
         char  *tb = temp_path_buf;
         size_t tblen = temp_path_buf_len;
         title = "dwarf_init_path_dl fails.";
@@ -1214,16 +1182,21 @@ process_one_file(
     }
     if (dres == DW_DLV_NO_ENTRY) {
         if (glflags.group_number > 0) {
+            struct esb_s m;
+
+            esb_constructor(&m);
+            homeify((char *)file_name,&m);
             printf("No DWARF information present in %s "
                 "for section group %d \n",
-                file_name,glflags.group_number);
+                sanitized(esb_get_string(&m)),glflags.group_number);
+            esb_destructor(&m);
         } else {
             struct esb_s m;
 
             esb_constructor(&m);
             homeify((char *)file_name,&m);
             printf("No DWARF information present in %s\n",
-                esb_get_string(&m));
+                sanitized(esb_get_string(&m)));
             esb_destructor(&m);
         }
         return dres;
@@ -1286,8 +1259,13 @@ process_one_file(
             /* path_source = DW_PATHSOURCE_basic; */
         }
         if (dres == DW_DLV_NO_ENTRY) {
+            struct esb_s m;
+
+            esb_constructor(&m);
+            homeify((char *)tied_file_name,&m);
             printf("No DWARF information present in tied file: %s\n",
-                tied_file_name);
+                sanitized(esb_get_string(&m)));
+            esb_destructor(&m);
             return dres;
         }
         if (dres == DW_DLV_ERROR) {
@@ -1407,7 +1385,8 @@ process_one_file(
         update_section_flags_per_groups();
     }
     reset_overall_CU_error_data();
-    if (glflags.gf_info_flag || glflags.gf_line_flag ||
+    if (glflags.gf_print_all_srcfiles ||
+        glflags.gf_info_flag || glflags.gf_line_flag ||
         glflags.gf_types_flag ||
         glflags.gf_check_macros || glflags.gf_macinfo_flag ||
         glflags.gf_macro_flag ||
@@ -1845,6 +1824,14 @@ process_one_file(
             pref == Dwarf_Alloc_Mmap?  "Dwarf_Alloc_Mmap":
             "<Unknown. an ERROR");
     }
+    if (glflags.gf_print_all_srcfiles) {
+        dd_print_all_srcfiles();
+        dd_destroy_all_srcfiles();
+    }
+
+    if (glflags.gf_print_language_version_table) {
+        print_language_version_table();
+    }
     if (glflags.gf_debug_addr_missing) {
         printf("\nERROR: At some point "
             "the .debug_addr section was needed but missing, "
