@@ -42,10 +42,14 @@ Portions Copyright (C) 2007-2020 David Anderson. All Rights Reserved.
 
 #include <stdlib.h> /* exit() */
 #include <string.h> /* strlen() */
+#include <stdio.h> /* FILE decl for dd_esb.h, printf etc */
 
 #include "dwarf.h"
 #include "libdwarf.h"
 #include "libdwarf_private.h"
+#include "dd_defined_types.h"
+#include "dd_checkutil.h"
+#include "dd_glflags.h"
 #include "dd_globals.h"
 #include "dd_dwconf.h"
 #include "dd_dwconf_using_functions.h"
@@ -55,11 +59,14 @@ Portions Copyright (C) 2007-2020 David Anderson. All Rights Reserved.
 #include "dd_addrmap.h"
 #include "dd_naming.h"
 #include "dd_safe_strcpy.h"
+#include "dd_glflags.h"
 #include "print_frames.h"
 
 #define TRUE 1
 #define FALSE 0
 
+#undef TESTOLDDW_OFFSET /* define only for a test. */
+
 Dwarf_Sig8 zero_type_signature;
 static void
 print_one_frame_reg_col(Dwarf_Debug dbg,
@@ -139,12 +146,12 @@ load_CU_error_data(Dwarf_Debug dbg,Dwarf_Die cu_die)
     }
 
     /* The offsets will be zero if it fails. Let it pass. */
-    atres = dwarf_die_offsets(cu_die,&glflags.DIE_overall_offset,
+    atres = dwarf_die_offsets(cu_die,&glflags.DIE_section_offset,
         &glflags.DIE_offset,&loadcuerr);
-    cu_die_goff = glflags.DIE_overall_offset;
+    cu_die_goff = glflags.DIE_section_offset;
     DROP_ERROR_INSTANCE(dbg,atres,loadcuerr);
 
-    glflags.DIE_CU_overall_offset = glflags.DIE_overall_offset;
+    glflags.DIE_CU_overall_offset = glflags.DIE_section_offset;
     glflags.DIE_CU_offset = glflags.DIE_offset;
     for (i = 0; i < atcnt; i++) {
         Dwarf_Half attr = 0;
@@ -191,7 +198,6 @@ load_CU_error_data(Dwarf_Debug dbg,Dwarf_Die cu_die)
 
             esb_constructor(&namestr);
             ares = get_attr_value(dbg, tag, cu_die,
-                /* die_indent_level */ 0,
                 cu_die_goff,attrib, srcfiles, srccnt,
                 &namestr, local_show_form_used,local_verbose,
                 &loadcuerr);
@@ -240,7 +246,7 @@ get_abstract_origin_funcname(Dwarf_Debug dbg,
     Dwarf_Error err = 0;
     Dwarf_Bool is_info = dwarf_get_die_infotypes_flag(die);
 
-    res = dwarf_global_formref(attr,&off,&err);
+    res = dwarf_global_formref_b(attr,&off,&is_info,&err);
     if (res == DW_DLV_ERROR) {
         dwarf_dealloc(dbg,err,DW_DLA_ERROR);
         return DW_DLV_NO_ENTRY;
@@ -377,6 +383,7 @@ get_proc_name_by_die(Dwarf_Debug dbg,
             simple_err_only_return_action(ares,
                 esb_get_string(&m));
             esb_destructor(&m);
+            dealloc_local_atlist(dbg,atlist,atcnt);
             return DW_DLV_ERROR;
         } else if (ares == DW_DLV_OK) {
             Dwarf_Error aterr = 0;
@@ -431,8 +438,9 @@ get_proc_name_by_die(Dwarf_Debug dbg,
                             dwarf_errmsg(aterr));
                         /* the long name is horrible */
                         if (!glflags.gf_error_code_search_by_address){
+                            /* error codes never big numbers. */
                             glflags.gf_error_code_search_by_address=
-                                dwarf_errno(aterr);
+                                (int)dwarf_errno(aterr);
                         }
                     }
                     dwarf_dealloc(dbg,aterr,DW_DLA_ERROR);
@@ -604,20 +612,24 @@ load_nested_proc_name(Dwarf_Debug dbg, Dwarf_Die die,
         /* try next sibling */
         prev_child = curdie;
         esb_empty_string(&nestname);
+#ifdef ORIGINAL_HEADER_API
         chres = dwarf_siblingof_b(dbg, curdie,
             dwarf_get_die_infotypes_flag(curdie),
             &newsibling, err);
+#else
+        chres = dwarf_siblingof_c(curdie,&newsibling,err);
+#endif /* ORIGINAL_HEADER_API */
         if (chres == DW_DLV_ERROR) {
             struct esb_s m;
 
             load_CU_error_data(dbg,*cu_die_for_print_frames);
             esb_constructor(&m);
             esb_append(&m,
-                "\nERROR: Looking for function name for "
+                "Looking for function name for "
                 "a frame. "
                 "dwarf_siblingof failed"
                 " trying to get the name. ");
-            print_error_and_continue(dbg,
+            print_error_and_continue(
                 esb_get_string(&m), chres,*err);
             esb_destructor(&m);
             DROP_ERROR_INSTANCE(dbg,chres,*err);
@@ -633,6 +645,7 @@ load_nested_proc_name(Dwarf_Debug dbg, Dwarf_Die die,
                 /*  If we got this die from the parent, we do not want
                     to dealloc here! */
                 dwarf_dealloc(dbg, prev_child, DW_DLA_DIE);
+                prev_child = 0;
             }
             /* Not there at this level */
             esb_destructor(&nestname);
@@ -644,8 +657,10 @@ load_nested_proc_name(Dwarf_Debug dbg, Dwarf_Die die,
             /*  If we got this die from the parent, we do not want
                 to dealloc here! */
             dwarf_dealloc(dbg, prev_child, DW_DLA_DIE);
+            /*  Clearing prev_child here gets coverity 330895
+                unused value warning.
+                prev_child = 0; */
         }
-        prev_child = 0;
         die_locally_gotten = 1;
     }
     if (die_locally_gotten) {
@@ -704,8 +719,9 @@ get_fde_proc_name_by_address(Dwarf_Debug dbg, Dwarf_Addr low_pc,
     }
     if (*cu_die_for_print_frames == NULL) {
         /* Call depends on dbg->cu_context to know what to do. */
-        cures = dwarf_next_cu_header_d(dbg,
+        cures = dwarf_next_cu_header_e(dbg,
             is_info,
+            cu_die_for_print_frames,
             &cu_header_length,
             &version_stamp, &abbrev_offset,
             &address_size,
@@ -725,6 +741,7 @@ get_fde_proc_name_by_address(Dwarf_Debug dbg, Dwarf_Addr low_pc,
                 "/procedure name. Section %s. Err is %s\n",
                 sanitized(frame_section_name),
                 dwarf_errmsg(*err));
+                glflags.gf_count_major_errors++;
             glflags.gf_all_cus_seen_search_by_address = 1;
             DROP_ERROR_INSTANCE(dbg,cures,*err);
             return DW_DLV_NO_ENTRY;
@@ -732,6 +749,7 @@ get_fde_proc_name_by_address(Dwarf_Debug dbg, Dwarf_Addr low_pc,
             /* loop thru the list again */
             *cu_die_for_print_frames = 0;
         } else {                /* DW_DLV_OK */
+#ifdef ORIGINAL_HEADER_API
             dres = dwarf_siblingof_b(dbg, NULL,
                 is_info,
                 cu_die_for_print_frames,
@@ -755,6 +773,7 @@ get_fde_proc_name_by_address(Dwarf_Debug dbg, Dwarf_Addr low_pc,
                 /*  No initial die? Something is wrong! */
                 return dres;
             }
+#endif /* ORIGINAL_HEADER_API */
         }
     }
     if (dres == DW_DLV_OK) {
@@ -799,6 +818,8 @@ get_fde_proc_name_by_address(Dwarf_Debug dbg, Dwarf_Addr low_pc,
         Dwarf_Die ldie = 0;
 
         type_signature = zero_type_signature;
+#ifdef ORIGINAL_HEADER_API
+
         cures = dwarf_next_cu_header_d(dbg,
             is_info, &cu_header_length,
             &version_stamp, &abbrev_offset,
@@ -808,6 +829,17 @@ get_fde_proc_name_by_address(Dwarf_Debug dbg, Dwarf_Addr low_pc,
             &next_cu_offset,
             &header_cu_type,
             err);
+#else
+        cures = dwarf_next_cu_header_e(dbg,
+            is_info, &ldie, &cu_header_length,
+            &version_stamp, &abbrev_offset,
+            &address_size,
+            &length_size,&extension_size,
+            &type_signature,&typeoffset,
+            &next_cu_offset,
+            &header_cu_type,
+            err);
+#endif
         if (cures != DW_DLV_OK) {
             if (cures == DW_DLV_ERROR) {
                 printf("\nERROR: Error getting "
@@ -824,11 +856,16 @@ get_fde_proc_name_by_address(Dwarf_Debug dbg, Dwarf_Addr low_pc,
             break;
         }
 
+#ifdef ORIGINAL_HEADER_API
         dres = dwarf_siblingof_b(dbg,NULL,is_info, &ldie, err);
+#else
+        dres = DW_DLV_OK;
+#endif
         if (*cu_die_for_print_frames) {
             dwarf_dealloc(dbg, *cu_die_for_print_frames,DW_DLA_DIE);
             *cu_die_for_print_frames = 0;
         }
+#ifdef ORIGINAL_HEADER_API
         if (dres == DW_DLV_ERROR) {
             DROP_ERROR_INSTANCE(dbg,dres,*err);
             glflags.gf_all_cus_seen_search_by_address = 1;
@@ -836,6 +873,7 @@ get_fde_proc_name_by_address(Dwarf_Debug dbg, Dwarf_Addr low_pc,
         } else if (dres == DW_DLV_NO_ENTRY) {
             return dres;
         }
+#endif /* ORIGINAL_HEADER_API */
         /*  DW_DLV_OK
             In normal processing (ie, when doing print_info()
             we would call print_attribute for each die
@@ -910,6 +948,98 @@ valid_fde_content(Dwarf_Small * fde_start,
     return TRUE;
 }
 
+static void
+dd_check_fde_cie(Dwarf_Debug dbg,
+    Dwarf_Fde fde,
+    Dwarf_Signed cie_index,
+    Dwarf_Cie cie)
+{
+    Dwarf_Error  error;
+    Dwarf_Cie    ret_cie = 0;
+    Dwarf_Signed ret_index = 0;
+    int          res = 0;
+
+    if (!glflags.gf_check_functions) {
+        return;
+    }
+    DWARF_CHECK_COUNT(check_functions_result,1);
+    res = dwarf_get_cie_of_fde(fde,&ret_cie,&error);
+    if (res == DW_DLV_ERROR) {
+        DWARF_CHECK_ERROR2(check_functions_result,
+            "Error calling dwarf_get_cie_of_fde() "
+            "underlying error is:",
+            dwarf_errmsg(error));
+        DROP_ERROR_INSTANCE(dbg,res,error);
+    } else if (res == DW_DLV_NO_ENTRY) {
+        DWARF_CHECK_ERROR(check_functions_result,
+            "NO_ENTRY calling dwarf_get_cie_of_fde()");
+    } else  { /* OK */
+        if ( ret_cie != cie) {
+            DWARF_CHECK_ERROR(check_functions_result,
+                "dwarf_get_cie_of_fde() returns "
+                "inconsistent cie");
+        }
+    }
+    res = dwarf_get_cie_index(cie,&ret_index,&error);
+    if (res == DW_DLV_ERROR) {
+        DWARF_CHECK_ERROR2(check_functions_result,
+            "Error calling dwarf_get_index() "
+            "underlying error is:",
+            dwarf_errmsg(error));
+        DROP_ERROR_INSTANCE(dbg,res,error);
+    } else if (res == DW_DLV_NO_ENTRY) {
+        DWARF_CHECK_ERROR(check_functions_result,
+            "NO_ENTRY calling dwarf_get_cie_index()");
+    } else  { /* OK */
+        if ( ret_index != cie_index) {
+            DWARF_CHECK_ERROR(check_functions_result,
+                "dwarf_get_cie_index() returns"
+                " mismatched cie index");
+        }
+    }
+}
+
+static void
+dd_check_retrieved_fde(Dwarf_Debug dbg,
+    Dwarf_Fde *fde_data,
+    Dwarf_Fde  expected_fde,
+    Dwarf_Addr pc)
+{
+    int         res = 0;
+    Dwarf_Error error = 0;
+    Dwarf_Fde   returned_fde = 0;
+    Dwarf_Addr  lopc = 0;
+    Dwarf_Addr  hipc = 0;
+
+    DWARF_CHECK_COUNT(check_functions_result,1);
+    res = dwarf_get_fde_at_pc(fde_data,pc,
+        &returned_fde,&lopc,&hipc,&error);
+    if (res == DW_DLV_ERROR) {
+        DWARF_CHECK_ERROR2(check_functions_result,
+            "Error calling dwarf_get_fde_at_pc() "
+            "underlying error is:",
+            dwarf_errmsg(error));
+        DROP_ERROR_INSTANCE(dbg,res,error);
+    } else if (res == DW_DLV_NO_ENTRY) {
+        struct esb_s m;
+
+        esb_constructor(&m);
+        esb_append_printf_u(&m,
+            "NO_ENTRY calling dwarf_get_fde_at_pc()"
+            " got NO ENTRY for pc 0x%"
+            DW_PR_XZEROS DW_PR_DUx
+            " which indicates corrupt dwarf",pc);
+        DWARF_CHECK_ERROR(check_functions_result,
+            esb_get_string(&m));
+        esb_destructor(&m);
+    } else  { /* OK */
+        if ( returned_fde != expected_fde) {
+            DWARF_CHECK_ERROR(check_functions_result,
+                "dwarf_get_fde_at_pc() returns"
+                " mismatched fde index");
+        }
+    }
+}
 static const Dwarf_Block blockzero;
 
 /*  Gather the fde print logic here so the control logic
@@ -917,8 +1047,8 @@ static const Dwarf_Block blockzero;
 static int
 print_one_fde(Dwarf_Debug dbg,
     const char *frame_section_name,
-    Dwarf_Fde  fde,
-    Dwarf_Unsigned fde_index,
+    Dwarf_Fde  *fde_data,
+    Dwarf_Signed fde_index,
     Dwarf_Cie * cie_data,
     Dwarf_Signed cie_element_count,
     Dwarf_Half address_size,
@@ -931,22 +1061,26 @@ print_one_fde(Dwarf_Debug dbg,
     Dwarf_Die *cu_die_for_print_frames,
     Dwarf_Error *err)
 {
-    Dwarf_Addr j = 0;
-    Dwarf_Addr low_pc = 0;
+    Dwarf_Addr     j = 0;
+    Dwarf_Addr     low_pc = 0;
     Dwarf_Unsigned func_length = 0;
-    Dwarf_Addr end_func_addr = 0;
-    Dwarf_Small *fde_bytes = NULL;
+    Dwarf_Addr     end_func_addr = 0;
+    Dwarf_Small   *fde_bytes = NULL;
     Dwarf_Unsigned fde_bytes_length = 0;
-    Dwarf_Off cie_offset = 0;
-    Dwarf_Signed cie_index = 0;
-    Dwarf_Off fde_offset = 0;
-    Dwarf_Signed eh_table_offset = 0;
-    int fres = 0;
-    int offres = 0;
-    struct esb_s temps;
-    int printed_intro_addr = 0;
-    char local_buf[100];
-    char temps_buf[200];
+    Dwarf_Off      cie_offset = 0;
+    Dwarf_Signed   cie_index = 0;
+    Dwarf_Off      fde_offset = 0;
+    Dwarf_Signed   eh_table_offset = 0;
+    int            fres = 0;
+    int            offres = 0;
+    struct esb_s   temps;
+    int            printed_intro_addr = 0;
+    char           local_buf[100];
+    char           temps_buf[200];
+    Dwarf_Fde      fde = 0;
+    unsigned long  fde_checking_count = 0;
+
+    fde = fde_data[fde_index];
     fres = dwarf_get_fde_range(fde,
         &low_pc, &func_length,
         &fde_bytes,
@@ -963,13 +1097,14 @@ print_one_fde(Dwarf_Debug dbg,
     if (fres == DW_DLV_NO_ENTRY) {
         return DW_DLV_NO_ENTRY;
     }
+    dd_check_fde_cie(dbg,fde,cie_index,cie_data[cie_index]);
     if (glflags.gf_cu_name_flag &&
         glflags.fde_offset_for_cu_low != DW_DLV_BADOFFSET &&
         (fde_offset < glflags.fde_offset_for_cu_low ||
         fde_offset > glflags.fde_offset_for_cu_high)) {
         return DW_DLV_NO_ENTRY;
     }
-    /* eh_table_offset is IRIX ONLY. */
+    /* eh_table_offset was IRIX ONLY. */
     fres = dwarf_get_fde_exception_info(fde,
         &eh_table_offset, err);
     if (fres == DW_DLV_ERROR) {
@@ -1063,7 +1198,7 @@ print_one_fde(Dwarf_Debug dbg,
     }
     esb_destructor(&temps);
     if (!is_eh) {
-        /* IRIX uses eh_table_offset. No one else uses it. */
+        /* IRIX used eh_table_offset. No one else uses it. */
         /* Do not print if in check mode */
         if (glflags.gf_do_print_dwarf) {
             if (eh_table_offset == DW_DLX_NO_EH_OFFSET) {
@@ -1128,11 +1263,21 @@ print_one_fde(Dwarf_Debug dbg,
         Dwarf_Addr cur_pc_in_table = 0;
 
         cur_pc_in_table = j;
+        if (glflags.gf_check_functions &&
+            fde_checking_count < 100){ /* 100 is arbitrary */
+            ++fde_checking_count;
+            dd_check_retrieved_fde(dbg,fde_data,fde,cur_pc_in_table);
+        }
         {
             Dwarf_Unsigned reg = 0;
             Dwarf_Unsigned offset_relevant = 0;
             Dwarf_Small  value_type = 0;
-            Dwarf_Unsigned offset = 0;
+            /*  DWARF has always said the offset is signed
+                in an offset(n) or val_offset(N) register rule.
+                the API functions here are formally
+                incorrect. So we cast the value returned
+                to us to Dwarf_Signed at point of use. */
+            Dwarf_Signed offset = 0;
             Dwarf_Block block;
             Dwarf_Addr   row_pc = 0;
             Dwarf_Bool   has_more_rows = 0;
@@ -1140,7 +1285,24 @@ print_one_fde(Dwarf_Debug dbg,
             int fires = 0;
 
             block = blockzero;
+#ifdef TESTOLDDW_OFFSET
+            {
+            Dwarf_Unsigned s = 0;
             fires = dwarf_get_fde_info_for_cfa_reg3_b(fde,
+                j,
+                &value_type,
+                &offset_relevant,
+                &reg,
+                &s,
+                &block,
+                &row_pc,
+                &has_more_rows,
+                &subsequent_pc,
+                err);
+            offset = (Dwarf_Signed)s;
+            }
+#else
+            fires = dwarf_get_fde_info_for_cfa_reg3_c(fde,
                 j,
                 &value_type,
                 &offset_relevant,
@@ -1151,12 +1313,13 @@ print_one_fde(Dwarf_Debug dbg,
                 &has_more_rows,
                 &subsequent_pc,
                 err);
+#endif
             if (fires == DW_DLV_ERROR) {
                 glflags.gf_count_major_errors++;
                 printf("\nERROR: on getting fde details for "
                     "fde row for address 0x%"
-                    DW_PR_XZEROS DW_PR_DUx "\n",
-                    j);
+                    DW_PR_XZEROS DW_PR_DUx ": %s\n",
+                    j,dwarf_errmsg(*err));
                 return fires;
             }
             if (fires == DW_DLV_NO_ENTRY) {
@@ -1194,14 +1357,27 @@ print_one_fde(Dwarf_Debug dbg,
             int fires = 0;
             Dwarf_Small value_type = 0;
             Dwarf_Block block;
-            Dwarf_Unsigned offset = 0;
+            Dwarf_Signed offset = 0;
             Dwarf_Addr row_pc = 0;
             Dwarf_Bool has_more_rows = FALSE;
             Dwarf_Addr subsequent_pc = 0;
 
             block = blockzero;
             {
+#if TESTOLDDW_OFFSET
+                Dwarf_Unsigned s = 0;
                 fires = dwarf_get_fde_info_for_reg3_b(fde,
+                    k,
+                    cur_pc_in_table,
+                    &value_type,
+                    &offset_relevant,
+                    &reg,
+                    &s,
+                    &block,
+                    &row_pc, &has_more_rows,&subsequent_pc, err);
+                offset = (Dwarf_Signed)s;
+#else
+                fires = dwarf_get_fde_info_for_reg3_c(fde,
                     k,
                     cur_pc_in_table,
                     &value_type,
@@ -1210,6 +1386,7 @@ print_one_fde(Dwarf_Debug dbg,
                     &offset,
                     &block,
                     &row_pc, &has_more_rows,&subsequent_pc, err);
+#endif
             }
             if (fires == DW_DLV_ERROR) {
                 printf("\n");
@@ -1272,12 +1449,12 @@ print_one_fde(Dwarf_Debug dbg,
             return offres;
         }
         if (offres == DW_DLV_NO_ENTRY) {
+            glflags.gf_count_major_errors++;
             printf("\nERROR:  Impossible, no fde offset or "
                 " cie offset for "
                 "fde index "
                 "%" DW_PR_DUu "?\n",
                 fde_index);
-            glflags.gf_count_major_errors++;
             return offres;
         }
         {
@@ -1388,6 +1565,7 @@ print_one_fde(Dwarf_Debug dbg,
                     cie_offset_size,
                     cie_version, config_data);
                 dwarf_dealloc_frame_instr_head(ihead);
+                ihead = 0;
             }
         }
     }
@@ -1549,6 +1727,7 @@ print_one_cie(Dwarf_Debug dbg,
                 address_size,
                 offset_size,version,config_data);
             dwarf_dealloc_frame_instr_head(ihead);
+            ihead = 0;
         }
     }
     return DW_DLV_OK;
@@ -1568,7 +1747,6 @@ print_expression_operations(Dwarf_Debug dbg,
 {
     Dwarf_Unsigned ulistlen = 0;
     int res2 = 0;
-    Dwarf_Addr baseaddr = 0; /* Really unknown */
     /* See PRINTING_DIES macro in print_die.c */
 
     {
@@ -1615,13 +1793,13 @@ print_expression_operations(Dwarf_Debug dbg,
             &locdesc_offset,
             err);
         if (lres == DW_DLV_ERROR) {
-            dwarf_loc_head_c_dealloc(head);
+            dwarf_dealloc_loc_head_c(head);
             glflags.gf_count_major_errors++;
             printf("\nERROR: calling dwarf_get_locdesc_entry_d()"
                 " on LocDesc 0");
             return lres;
         } else if (lres == DW_DLV_NO_ENTRY) {
-            dwarf_loc_head_c_dealloc(head);
+            dwarf_dealloc_loc_head_c(head);
             return lres;
         }
         /*  ASSERT: loclist_source == DW_LKIND_expression  */
@@ -1630,13 +1808,9 @@ print_expression_operations(Dwarf_Debug dbg,
             die,
             die_indent_level,
             locentry,
-            0, /* index 0: locdesc 0 */
             ulocentry_count,
-            DW_LKIND_expression /* loclist_source */,
-            0, /* no die indent*/
-            baseaddr,
             out_string,err);
-        dwarf_loc_head_c_dealloc(head);
+        dwarf_dealloc_loc_head_c(head);
         return lres;
     }
 }
@@ -1659,51 +1833,6 @@ lastop_pointless(int op)
     return FALSE;
 }
 
-#if 0
-/*  iregion_start, iregion_end are the overall
-    block of fde/cie instructions.
-    idata, idata end are the area next to be read
-    and they must lie within the iregion* range.
-    The end address is one past the last byte.
-
-    We are decoding here, libdwarf has
-    not decoded these bytes, so it is up to us to
-    check for corrupt data in the frame section.
-*/
-static int
-check_finstr_addrs(unsigned char *iregionstart,
-    unsigned char *idata,
-    unsigned char *idata_end,
-    unsigned char *iregionend,
-    const char *msg)
-{
-    if ( idata > idata_end) {
-        /* zero length allowed.  But maybe overflow happened. */
-        glflags.gf_count_major_errors++;
-        printf("ERROR: frame instruction internal error reading %s\n",
-            msg);
-        return DW_DLV_ERROR;
-    }
-    if (idata < iregionstart) {
-        glflags.gf_count_major_errors++;
-        printf("\nERROR: frame instruction overflow(?) reading"
-            "  %s\n", msg);
-        return DW_DLV_ERROR;
-    }
-    if (idata_end > iregionend) {
-        Dwarf_Unsigned bytes_in = 0;
-        bytes_in = idata - iregionstart;
-        glflags.gf_count_major_errors++;
-        printf("\nERROR: frame instruction reads off end"
-            " %" DW_PR_DUu
-            " bytes into instructions for %s\n",
-            bytes_in,msg);
-        return DW_DLV_ERROR;
-    }
-    return DW_DLV_OK;
-}
-#endif
-
 static char exprstr_buf[1000];
 static int
 print_expression( Dwarf_Debug dbg,
@@ -1953,11 +2082,6 @@ print_frame_inst_bytes(Dwarf_Debug dbg,
         default:
             printf("UNKNOWN FIELD 0x%x\n",fields[0]);
         }
-#if 0
-        glflags.gf_count_major_errors++;
-        printf("\nERROR: Unexpected frame instruction string"
-            " \"%s\" not understood.\n",fields);
-#endif
     }
     if (lastop_pointless(lastop)) {
         printf(
@@ -2040,9 +2164,15 @@ print_one_frame_reg_col(Dwarf_Debug dbg,
         if (print_type_title) {
             printf("<%s ", type_title);
         }
+        if (offset_relevant) {
+            printf("ERROR: frame register type %s "
+                "marked as offset_relevant "
+                "but that is impossible, it has no such "
+                "field.",type_title);
+            glflags.gf_count_major_errors++;
+        }
         printreg(rule_id, config_data);
         printf("=");
-        /*  Here 'offset' is actually block length. */
         printf("expr-block-len=%" DW_PR_DUu , block->bl_len);
         if (print_type_title) {
             printf("> ");
@@ -2055,7 +2185,7 @@ print_one_frame_reg_col(Dwarf_Debug dbg,
                 libdwarf so libdwarf validated it. */
             dump_block("", block->bl_data, block->bl_len);
             printf("> ");
-            if (glflags.verbose) {
+            {
                 struct esb_s exprstring;
                 char local_buf[300];
                 int gres = 0;
@@ -2063,7 +2193,6 @@ print_one_frame_reg_col(Dwarf_Debug dbg,
 
                 esb_constructor_fixed(&exprstring,local_buf,
                     sizeof(local_buf));
-                /*  Here 'offset' is actually block length. */
                 gres = print_expression_operations(dbg,
                     die,
                     /* indent */ 1,
@@ -2095,11 +2224,42 @@ print_one_frame_reg_col(Dwarf_Debug dbg,
     default:
         printf("Internal error in libdwarf, value type %d\n",
             value_type);
-        exit(1);
+        exit(EXIT_FAILURE);
     }
     return;
 }
 
+static void
+dd_check_get_fde_n(Dwarf_Debug dbg,
+    Dwarf_Fde *fde_data,
+    Dwarf_Fde expected_fde,
+    Dwarf_Signed index)
+{
+    int         res = 0;
+    Dwarf_Error localerr = 0;
+    Dwarf_Fde   ret_fde = 0;
+
+    DWARF_CHECK_COUNT(check_functions_result,1);
+    res = dwarf_get_fde_n(fde_data,
+        (Dwarf_Unsigned)index,&ret_fde,&localerr);
+    if (res == DW_DLV_ERROR) {
+        DWARF_CHECK_ERROR2(check_functions_result,
+            "dwarf_get_cie_index() returned DW_DLV_ERRORi "
+            "underlying error is ",
+            dwarf_errmsg(localerr));
+        DROP_ERROR_INSTANCE(dbg,res,localerr);
+    } else if (res == DW_DLV_NO_ENTRY) {
+        DWARF_CHECK_ERROR(check_functions_result,
+            "dwarf_get_cie_index() returned DW_DLV_NO_ENTRY ");
+    } else {
+        if (ret_fde != expected_fde) {
+            DWARF_CHECK_ERROR(check_functions_result,
+                "dwarf_get_cie_index() does not"
+                " return the expected fde!");
+        }
+    }
+}
+
 /*  We do NOT want to free cie/fde data as we will
     use that in print_all_cies() */
 static int
@@ -2143,10 +2303,17 @@ print_all_fdes(Dwarf_Debug dbg,
 
     for (i = 0; i < fde_element_count; i++) {
         int fdres = 0;
+        Dwarf_Fde fde = 0;
 
+        fde = fde_data[i];
+        if (glflags.gf_check_functions) {
+            dd_check_get_fde_n(dbg,fde_data,
+                fde,i);
+        }
         fdres = print_one_fde(dbg,
-            frame_section_name, fde_data[i],
-            i, cie_data, cie_element_count,
+            frame_section_name,
+            fde_data,i,
+            cie_data, cie_element_count,
             address_size, offset_size, version,
             is_eh, config_data,
             map_lowpc_to_name,
