@@ -12,37 +12,38 @@ for anyone to use for any purpose.
         make
         ./frame1 frame1
 
+    This uses public structs from libdwarf.h so it is
+    stuck with limitations for compatibility.
+
+    There needs to be a more flexible alternative, one
+    can be updated when DWARF changes.
+
     gcc/clang may produce .eh_frame without .debug_frame.
     To read .eh_frame call dwarf_get_fde_list_eh()
     below instead of dwarf_get_fde_list() .
 */
-#include "config.h"
 
-/* Windows specific header files */
-#if defined(_WIN32) && defined(HAVE_STDAFX_H)
-#include "stdafx.h"
-#endif /* HAVE_STDAFX_H */
+#include <config.h>
+
+#include <stddef.h> /* NULL */
+#include <stdio.h>  /* printf() snprintf() */
+#include <stdlib.h> /* exit() free() malloc() */
+#include <string.h> /* strcat() strcmp() strcpy() strncmp() */
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h> /* For open() */
-#endif /* HAVE_SYS_TYPES_H */
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>  /* For open() */
-#endif /* HAVE_SYS_STAT_H */
-#include <fcntl.h>     /* For open() */
-#include <stdlib.h> /* for exit(), C89 malloc */
-#ifdef HAVE_MALLOC_H
-/* Useful include for some Windows compilers. */
-#include <malloc.h>
-#endif /* HAVE_MALLOC_H */
 #ifdef _WIN32
-#include <io.h> /* close() */
+#include <io.h> /* close() open() */
 #elif defined HAVE_UNISTD_H
 #include <unistd.h> /* close() */
 #endif /* _WIN32 */
-#ifdef HAVE_STDINT_H
-#include <stdint.h> /* For uintptr_t */
-#endif /* HAVE_STDINT_H */
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h> /* open() O_RDONLY */
+#endif /* HAVE_FCNTL_H */
+
+#include "dwarf.h"
+#include "libdwarf.h"
+#include "libdwarf_private.h"
+
 #ifdef _O_BINARY
 /*  This is for a Windows environment */
 #define O_BINARY _O_BINARY
@@ -51,22 +52,23 @@ for anyone to use for any purpose.
 # define O_BINARY 0  /* So it does nothing in Linux/Unix */
 # endif
 #endif /* O_BINARY */
-#include <string.h>     /* For strcmp* */
-#include <stdio.h>
-#include <errno.h>
-#include "dwarf.h"
-#include "libdwarf.h"
-#include "libdwarf_private.h"
+static int print_frame_instrs(Dwarf_Debug dbg,
+    Dwarf_Frame_Instr_Head frame_instr_head,
+    Dwarf_Unsigned frame_instr_count,
+    Dwarf_Error *error);
 
 static void read_frame_data(Dwarf_Debug dbg,const char *sec);
 static void print_fde_instrs(Dwarf_Debug dbg, Dwarf_Fde fde,
     Dwarf_Error *error);
 static void print_regtable(Dwarf_Regtable3 *tab3);
-static void print_cie_instrs(Dwarf_Cie cie,Dwarf_Error *error);
+static void print_cie_instrs(Dwarf_Debug dbg,
+    Dwarf_Cie cie,Dwarf_Error *error);
 static void print_fde_selected_regs( Dwarf_Fde fde);
 static void print_reg(int r);
 
 static int just_print_selected_regs = 0;
+static int print_selected_regs = 0;
+static int stop_at_n_fdes = 0;
 
 /*  Depending on the ABI we set INITIAL_VAL
     differently.  For ia64 initial value is
@@ -74,10 +76,11 @@ static int just_print_selected_regs = 0;
     value is SAME_VAL.
     Here we'll set it UNDEF_VAL
     as that way we'll see when first set. */
-#define UNDEF_VAL 2000
-#define SAME_VAL 2001
-#define CFA_VAL 2002
-#define INITIAL_VAL UNDEF_VAL
+#define UNDEF_VAL DW_FRAME_UNDEFINED_VAL
+#define SAME_VAL DW_FRAME_SAME_VAL
+#define CFA_VAL DW_FRAME_CFA_COL
+/*#define INITIAL_VAL UNDEF_VAL */
+#define INITIAL_VAL SAME_VAL
 
 /* Dumping a dwarf-expression as a byte stream. */
 static void
@@ -112,6 +115,8 @@ main(int argc, char **argv)
     Dwarf_Ptr errarg = 0;
     int regtabrulecount = 0;
     int curopt = 0;
+    const char *countstr = "--stop-at-fde-n=";
+    int countstr_len = strlen(countstr);
 
     for (curopt = 1;curopt < argc; ++curopt) {
         if (strncmp(argv[curopt],"--",2)) {
@@ -119,6 +124,25 @@ main(int argc, char **argv)
         }
         if (!strcmp(argv[curopt],"--just-print-selected-regs")) {
             just_print_selected_regs++;
+            continue;
+        }
+        if (!strncmp(argv[curopt],countstr,countstr_len)) {
+            char *carg = argv[curopt];
+            if (!carg[countstr_len]) {
+                printf("Improper %s arg, needs a number. Ignored\n",
+                    carg);
+            } else {
+                stop_at_n_fdes = atoi((const char *)
+                    (carg+countstr_len));
+            }
+        }
+        if (!strcmp(argv[curopt],"--print-selected-regs")) {
+            print_selected_regs++;
+            continue;
+        }
+        if (!strcmp(argv[curopt],"--suppress-de-alloc-tree")) {
+            /* Do nothing, ignore the argument */
+            continue;
         }
     }
 
@@ -129,11 +153,11 @@ main(int argc, char **argv)
         fd = open(filepath,O_RDONLY|O_BINARY);
         if (fd < 0) {
             printf("Unable to open %s, giving up.\n",filepath);
-            exit(1);
+            exit(EXIT_FAILURE);
         }
     } else {
         printf("Too many args, giving up. \n");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
     res = dwarf_init_b(fd,DW_GROUPNUMBER_ANY,
         errhand,errarg, &dbg,&error);
@@ -143,7 +167,8 @@ main(int argc, char **argv)
         if (res == DW_DLV_ERROR) {
             printf("Error code %s\n",dwarf_errmsg(error));
         }
-        exit(1);
+        dwarf_dealloc_error(dbg,error);
+        exit(EXIT_FAILURE);
     }
     /*  Do this setting after init before any real operations.
         These return the old values, but here we do not
@@ -204,7 +229,7 @@ read_frame_data(Dwarf_Debug dbg,const char *sect)
     }
     if ( res == DW_DLV_ERROR) {
         printf("Error reading frame data ");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
     printf( "%" DW_PR_DSd " cies present. "
         "%" DW_PR_DSd " fdes present. \n",
@@ -212,25 +237,33 @@ read_frame_data(Dwarf_Debug dbg,const char *sect)
     /*if (fdenum >= fde_element_count) {
         printf("Want fde %d but only %" DW_PR_DSd " present\n",fdenum,
             fde_element_count);
-        exit(1);
+        exit(EXIT_FAILURE);
     }*/
 
     for (fdenum = 0; fdenum < fde_element_count; ++fdenum) {
         Dwarf_Cie cie = 0;
 
+        if (stop_at_n_fdes && fdenum >= stop_at_n_fdes) {
+            printf("\nStopping at %d FDEs by request\n",
+                stop_at_n_fdes);
+            break;
+        }
         res = dwarf_get_cie_of_fde(fde_data[fdenum],&cie,&error);
         if (res != DW_DLV_OK) {
             printf("Error accessing cie of fdenum %" DW_PR_DSd
                 " to get its cie\n",fdenum);
-            exit(1);
+            exit(EXIT_FAILURE);
         }
         printf("Print cie of fde %" DW_PR_DSd  "\n",fdenum);
-        print_cie_instrs(cie,&error);
-        printf("Print fde %" DW_PR_DSd  "\n",fdenum);
+        print_cie_instrs(dbg,cie,&error);
+        printf("\nPrint fde %" DW_PR_DSd  "\n",fdenum);
         if (just_print_selected_regs) {
             print_fde_selected_regs(fde_data[fdenum]);
         } else {
             print_fde_instrs(dbg,fde_data[fdenum],&error);
+            if (print_selected_regs) {
+                print_fde_selected_regs(fde_data[fdenum]);
+            }
         }
     }
 
@@ -242,7 +275,7 @@ read_frame_data(Dwarf_Debug dbg,const char *sect)
 
 /* Simply shows the instructions at hand for this fde. */
 static void
-print_cie_instrs(Dwarf_Cie cie,Dwarf_Error *error)
+print_cie_instrs(Dwarf_Debug dbg,Dwarf_Cie cie,Dwarf_Error *error)
 {
     int res = DW_DLV_ERROR;
     Dwarf_Unsigned bytes_in_cie = 0;
@@ -254,6 +287,7 @@ print_cie_instrs(Dwarf_Cie cie,Dwarf_Error *error)
     Dwarf_Small   *instrp = 0;
     Dwarf_Unsigned instr_len = 0;
     Dwarf_Half offset_size = 0;
+    Dwarf_Signed cie_index = 0;
 
     res = dwarf_get_cie_info_b(cie,&bytes_in_cie,
         &version, &augmentation, &code_alignment_factor,
@@ -261,7 +295,47 @@ print_cie_instrs(Dwarf_Cie cie,Dwarf_Error *error)
         &instrp,&instr_len,&offset_size,error);
     if (res != DW_DLV_OK) {
         printf("Unable to get cie info!\n");
-        exit(1);
+        exit(EXIT_FAILURE);
+    }
+    res = dwarf_get_cie_index(cie,&cie_index,error);
+    if (res != DW_DLV_OK) {
+        printf("Unable to get cie index!\n");
+        exit(EXIT_FAILURE);
+    }
+    printf("CIE info\n");
+    printf("  cie index              : %" DW_PR_DSd "\n",
+        cie_index);
+    printf("  cie length             : 0x%" DW_PR_DUx " (%"
+        DW_PR_DUu ")\n",
+        bytes_in_cie,bytes_in_cie);
+    printf("  cie version            : %d\n",version);
+    printf("  cie augmenter          : %s\n",
+        augmentation?augmentation:"<none>");
+    printf("  code alignment factor  : %" DW_PR_DUu "\n",
+        code_alignment_factor);
+    printf("  data alignment factor  : %" DW_PR_DSd "\n",
+        data_alignment_factor);
+    printf("  return address register: %u\n",
+        return_address_register_rule);
+    printf("  initial instructions length: %" DW_PR_DUu "\n",
+        instr_len);
+    printf("  offset size            : %u\n",offset_size);
+    {
+        Dwarf_Frame_Instr_Head frame_instr_head = 0;
+        Dwarf_Unsigned frame_instr_count = 0;
+        res = dwarf_expand_frame_instructions(cie,
+            instrp,instr_len,
+            &frame_instr_head,
+            &frame_instr_count,
+            error);
+        if (res != DW_DLV_OK) {
+            printf("dwarf_expand_frame_instructions failed!\n");
+            exit(EXIT_FAILURE);
+        }
+        printf("CIE op count: %" DW_PR_DUu "\n",frame_instr_count);
+        print_frame_instrs(dbg,frame_instr_head,
+            frame_instr_count, error);
+        dwarf_dealloc_frame_instr_head(frame_instr_head);
     }
 }
 
@@ -280,27 +354,28 @@ print_fde_col(Dwarf_Signed k,
     char *type_title = "";
     Dwarf_Unsigned rule_id = k;
 
-    printf(" pc=0x%" DW_PR_DUx ,jsave);
+    (void)has_more_rows;
+    (void)subsequent_pc;
     if (row_pc != jsave) {
         printf(" row_pc=0x%" DW_PR_DUx ,row_pc);
     }
     printf(" col=%" DW_PR_DSd " ",k);
     switch(value_type) {
     case DW_EXPR_OFFSET:
-        type_title = "off";
+        type_title = "DW_EXPR_OFFSET";
         goto preg2;
     case DW_EXPR_VAL_OFFSET:
-        type_title = "valoff";
+        type_title = "DW_EXPR_VAL_OFFSET";
 
         preg2:
         printf("<%s ", type_title);
         if (reg_used == SAME_VAL) {
             printf(" SAME_VAL");
-            break;
-        } else if (reg_used == INITIAL_VAL) {
-            printf(" INITIAL_VAL");
-            break;
+            /* break; */
+        } else if (reg_used == UNDEF_VAL) {
+            printf(" UNDEF_VAL");
         }
+        printf("[");
         print_reg(rule_id);
 
         printf("=");
@@ -313,12 +388,13 @@ print_fde_col(Dwarf_Signed k,
             print_reg(reg_used);
             printf(") ");
         }
+        printf("]");
         break;
     case DW_EXPR_EXPRESSION:
-        type_title = "expr";
+        type_title = "DW_EXPR_EXPRESSION";
         goto pexp2;
     case DW_EXPR_VAL_EXPRESSION:
-        type_title = "valexpr";
+        type_title = "DW_EXPR_VAL_EXPRESSION";
 
         pexp2:
         printf("<%s ", type_title);
@@ -351,11 +427,19 @@ print_fde_col(Dwarf_Signed k,
     default:
         printf("Internal error in libdwarf, value type %d\n",
             value_type);
-        exit(1);
+        exit(EXIT_FAILURE);
     }
-    printf(" more=%d",has_more_rows);
-    printf(" next=0x%" DW_PR_DUx,subsequent_pc);
-    printf("%s", "> ");
+#if 0
+    if (has_more_rows) {
+        printf(" has_more_rows? %s next pc: 0x%"
+            DW_PR_DUx ">",
+            has_more_rows?"yes.":"no.",
+            subsequent_pc);
+    } else {
+        printf("%s", ">");
+    }
+#endif
+    printf("%s", ">");
     printf("\n");
 }
 
@@ -376,7 +460,8 @@ print_fde_selected_regs( Dwarf_Fde fde)
 {
     Dwarf_Error oneferr = 0;
     /* Arbitrary column numbers for testing. */
-    static int selected_cols[] = {1,3,5};
+    static int selected_cols[] = {1,2,3,4,5,6,7,8,
+        9,10,11,12,13,14,15,16};
     static int selected_cols_count =
         sizeof(selected_cols)/sizeof(selected_cols[0]);
     Dwarf_Signed k = 0;
@@ -394,7 +479,7 @@ print_fde_selected_regs( Dwarf_Fde fde)
     Dwarf_Addr jsave = 0;
     Dwarf_Addr high_addr = 0;
     Dwarf_Addr next_jsave = 0;
-    Dwarf_Bool has_more_rows = 0;
+    Dwarf_Bool has_more_rows = FALSE;
     Dwarf_Addr subsequent_pc = 0;
     Dwarf_Error error = 0;
     int res = 0;
@@ -410,7 +495,7 @@ print_fde_selected_regs( Dwarf_Fde fde)
         printf("FAIL: dwarf_get_fde_range err %" DW_PR_DUu
             " line %d\n",
             dwarf_errno(oneferr),__LINE__);
-        exit(1);
+        exit(EXIT_FAILURE);
     }
     if (fres == DW_DLV_NO_ENTRY) {
         printf("No fde range data available\n");
@@ -419,7 +504,7 @@ print_fde_selected_regs( Dwarf_Fde fde)
     res = dwarf_get_cie_of_fde(fde,&cie,&error);
     if (res != DW_DLV_OK) {
         printf("Error getting cie from fde\n");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     high_addr = low_pc + func_length;
@@ -429,6 +514,8 @@ print_fde_selected_regs( Dwarf_Fde fde)
         jsave = next_jsave) {
         next_jsave = jsave+1;
         printf("\n");
+        printf(" FDE columns (registers) for pc 0x%"
+            DW_PR_DUx "\n",jsave);
         for (k = 0; k < selected_cols_count ; ++k ) {
             Dwarf_Unsigned reg = 0;
             Dwarf_Unsigned offset_relevant = 0;
@@ -454,7 +541,7 @@ print_fde_selected_regs( Dwarf_Fde fde)
             if (fires == DW_DLV_ERROR) {
                 printf("FAIL: reading reg err %" DW_PR_DUu " line %d",
                     dwarf_errno(oneferr),__LINE__);
-                exit(1);
+                exit(EXIT_FAILURE);
             }
             if (fires == DW_DLV_NO_ENTRY) {
                 continue;
@@ -523,7 +610,8 @@ print_frame_instrs(Dwarf_Debug dbg,
         switch(fields[0]) {
         case 'u': {
             if (!fields[1]) {
-                printf("%" DW_PR_DUu "\n",u0);
+                printf("%" DW_PR_DUu " (0x%" DW_PR_DUx "\n",
+                    u0,u0);
             }
             if (fields[1] == 'c') {
                 Dwarf_Unsigned final =
@@ -555,7 +643,7 @@ print_frame_instrs(Dwarf_Debug dbg,
                     Dwarf_Signed final =
                         (Dwarf_Signed)u0 *
                         data_alignment_factor;
-                    printf("%" DW_PR_DUu ,final);
+                    printf("%" DW_PR_DSd ,final);
                     printf("\n");
                 }
             }
@@ -654,6 +742,8 @@ print_fde_instrs(Dwarf_Debug dbg,
     Dwarf_Signed cie_index = 0;
     Dwarf_Off fde_offset = 0;
     Dwarf_Addr arbitrary_addr = 0;
+    Dwarf_Addr subsequent_pc = 0;
+    Dwarf_Bool has_more_rows =  TRUE;
     Dwarf_Addr actual_pc = 0;
     Dwarf_Regtable3 tab3;
     int oldrulecount = 0;
@@ -665,7 +755,7 @@ print_fde_instrs(Dwarf_Debug dbg,
         &fde_byte_length,&cie_offset,&cie_index,&fde_offset,error);
     if (res != DW_DLV_OK) {
         printf("Problem getting fde range \n");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     arbitrary_addr = lowpc + (func_length/2);
@@ -685,29 +775,60 @@ print_fde_instrs(Dwarf_Debug dbg,
         sizeof(struct Dwarf_Regtable_Entry3_s)* oldrulecount);
     if (!tab3.rt3_rules) {
         printf("Unable to malloc for %d rules\n",oldrulecount);
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     res = dwarf_get_fde_info_for_all_regs3(fde,arbitrary_addr ,
         &tab3,&actual_pc,error);
-    printf("function actual addr of row 0x%" DW_PR_DUx "\n",
-        actual_pc);
-
+    printf("function  Requested_pc 0x%"
+        DW_PR_DUx " Actual addr of row 0x%" DW_PR_DUx "\n",
+        arbitrary_addr,actual_pc);
     if (res != DW_DLV_OK) {
+        free(tab3.rt3_rules);
+        tab3.rt3_rules = 0;
         printf("dwarf_get_fde_info_for_all_regs3 failed!\n");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
-    print_regtable(&tab3);
+    /*  Now for an example of iterating through a range of addrs
+        efficiently, lets redo the above and iterate pc values.
+        the function called is new as of 0.9.0 October 2023. */
+    for (arbitrary_addr=lowpc; has_more_rows  ;
+        arbitrary_addr = subsequent_pc) {
+        res = dwarf_get_fde_info_for_all_regs3_b(fde,arbitrary_addr ,
+            &tab3,&actual_pc,&has_more_rows, &subsequent_pc,error);
+        if (res != DW_DLV_OK) {
+            free(tab3.rt3_rules);
+            tab3.rt3_rules = 0;
+            printf("dwarf_get_fde_info_for_all_regs3_b failed!\n");
+            exit(EXIT_FAILURE);
+        }
+#if 0
+            " More rows? %s Subsequent_pc 0x%" DW_PR_DUx "\n",
+            arbitrary_addr,actual_pc,has_more_rows?"yes":"no",
+            subsequent_pc);
+#endif
+        printf("\nRegtable at pc 0x%" DW_PR_DUx "\n",actual_pc);
+        print_regtable(&tab3);
+        if (has_more_rows) {
+            printf("  Next row to print is pc 0x%"
+                DW_PR_DUx "\n",subsequent_pc);
+        }
+    }
+    /*print_regtable(&tab3); */
 
     res = dwarf_get_fde_instr_bytes(fde,&outinstrs,&instrslen,error);
     if (res != DW_DLV_OK) {
+        free(tab3.rt3_rules);
+        tab3.rt3_rules = 0;
         printf("dwarf_get_fde_instr_bytes failed!\n");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
     res = dwarf_get_cie_of_fde(fde,&cie,error);
     if (res != DW_DLV_OK) {
+        free(tab3.rt3_rules);
+        tab3.rt3_rules = 0;
         printf("Error getting cie from fde\n");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     {
@@ -719,8 +840,10 @@ print_fde_instrs(Dwarf_Debug dbg,
             &frame_instr_count,
             error);
         if (res != DW_DLV_OK) {
+            free(tab3.rt3_rules);
+            tab3.rt3_rules = 0;
             printf("dwarf_expand_frame_instructions failed!\n");
-            exit(1);
+            exit(EXIT_FAILURE);
         }
         printf("Frame op count: %" DW_PR_DUu "\n",frame_instr_count);
         print_frame_instrs(dbg,frame_instr_head,
@@ -729,6 +852,7 @@ print_fde_instrs(Dwarf_Debug dbg,
         dwarf_dealloc_frame_instr_head(frame_instr_head);
     }
     free(tab3.rt3_rules);
+    tab3.rt3_rules = 0;
 }
 
 static void
@@ -736,59 +860,82 @@ print_reg(int r)
 {
     switch(r) {
     case SAME_VAL:
-        printf(" %d SAME_VAL ",r);
+        printf(" [DW_FRAME_SAME_VAL] ");
         break;
     case UNDEF_VAL:
-        printf(" %d UNDEF_VAL ",r);
+        printf(" [DW_FRAME_UNDEF_VAL] ");
         break;
     case CFA_VAL:
-        printf(" %d (CFA) ",r);
+        printf(" [(CFA)] ");
         break;
     default:
-        printf(" r%d ",r);
+        printf(" [r%d] ",r);
         break;
     }
 }
 
+static char *
+value_type_name(int valuetype,char *buf,unsigned buflen)
+{
+    buf[0] = 0;
+    switch(valuetype) {
+    case DW_EXPR_OFFSET:
+        return "DW_EXPR_OFFSET";
+    case DW_EXPR_VAL_OFFSET:
+        return "DW_EXPR_VAL_OFFSET";
+    case DW_EXPR_EXPRESSION:
+        return "DW_EXPR_EXPRESSION";
+    case DW_EXPR_VAL_EXPRESSION:
+        return "DW_EXPR_VAL_EXPRESSION";
+    default:
+        break;
+    }
+    snprintf(buf,buflen, "Unknown(%d)",valuetype);
+    return buf;
+}
+
 static void
-print_one_regentry(const char *prefix,
+print_one_regentry(const char *prefix_i,
     struct Dwarf_Regtable_Entry3_s *entry)
 {
+    char buf[100];
+    const char *prefix = prefix_i;
     int is_cfa = !strcmp("cfa",prefix);
+    if (is_cfa) {
+        prefix="cfa  ";
+    }
+
+    buf[0] = 0;
     printf("%s ",prefix);
-    printf("type: %d %s ",
-        entry->dw_value_type,
-        (entry->dw_value_type == DW_EXPR_OFFSET)? "DW_EXPR_OFFSET":
-        (entry->dw_value_type == DW_EXPR_VAL_OFFSET)?
-            "DW_EXPR_VAL_OFFSET":
-        (entry->dw_value_type == DW_EXPR_EXPRESSION)?
-            "DW_EXPR_EXPRESSION":
-        (entry->dw_value_type == DW_EXPR_VAL_EXPRESSION)?
-            "DW_EXPR_VAL_EXPRESSION":
-            "Unknown");
+    printf("type: [%s] ",
+        value_type_name(entry->dw_value_type,
+        buf,(unsigned)sizeof(buf)));
     switch(entry->dw_value_type) {
     case DW_EXPR_OFFSET:
         print_reg(entry->dw_regnum);
-        printf(" offset_rel? %d ",entry->dw_offset_relevant);
+        printf("   [offset_rel? %s ",
+            entry->dw_offset_relevant?"yes.":"no.");
         if (entry->dw_offset_relevant) {
-            printf(" offset  %" DW_PR_DSd " " ,
-                entry->dw_offset);
+            printf(" Offset  %" DW_PR_DSd " " ,
+                (Dwarf_Signed)entry->dw_offset);
             if (is_cfa) {
-                printf("defines cfa value");
+                printf("Defines cfa value");
             } else {
-                printf("address of value is CFA plus signed offset");
+                printf("Address of value is CFA plus signed offset");
             }
             if (!is_cfa  && entry->dw_regnum != CFA_VAL) {
                 printf(" compiler botch, regnum != CFA_VAL");
             }
         } else {
-            printf("value in register");
+            printf("Value in register");
         }
+        printf("]");
         break;
     case DW_EXPR_VAL_OFFSET:
         print_reg(entry->dw_regnum);
+        printf("[");
         printf(" offset  %" DW_PR_DSd " " ,
-            entry->dw_offset);
+            (Dwarf_Signed)entry->dw_offset);
         if (is_cfa) {
             printf("does this make sense? No?");
         } else {
@@ -797,23 +944,42 @@ print_one_regentry(const char *prefix,
         if (!is_cfa  && entry->dw_regnum != CFA_VAL) {
             printf(" compiler botch, regnum != CFA_VAL");
         }
+        printf("]");
         break;
     case DW_EXPR_EXPRESSION:
         print_reg(entry->dw_regnum);
-        printf(" offset_rel? %d ",entry->dw_offset_relevant);
-        printf(" offset  %" DW_PR_DUu " " ,
-            entry->dw_offset);
+        printf("[");
+        if (entry->dw_offset_relevant) {
+            printf(" FAIL. ERROR: a DW_EXPR_EXPRESSION "
+                "must not have the dw_offset marked as "
+                "offset_relevant \n");
+            printf(" offset_rel  ERROR: %d ",
+                entry->dw_offset_relevant);
+            printf(" offset  %" DW_PR_DSd " " ,
+                (Dwarf_Signed)entry->dw_offset);
+        }
         printf("Block ptr set? %s ",
             entry->dw_block.bl_data?"yes":"no");
         printf(" Value is at address given by expr val ");
         /* printf(" block-ptr  0x%" DW_PR_DUx " ",
             (Dwarf_Unsigned)entry->dw_block_ptr); */
+        printf("]");
         break;
     case DW_EXPR_VAL_EXPRESSION:
+        printf("[");
         printf(" expression byte len  %" DW_PR_DUu " " ,
             entry->dw_block.bl_len);
         printf("Block ptr set? %s ",
             entry->dw_block.bl_data?"yes":"no");
+        if (entry->dw_offset_relevant) {
+            printf(" FAIL. ERROR: a DW_EXPR_VAL_EXPRESSION "
+                "must not have the dw_offset marked as "
+                "offset_relevant \n");
+            printf(" offset_rel  ERROR: %d ",
+                entry->dw_offset_relevant);
+            printf(" offset  %" DW_PR_DSd " " ,
+                (Dwarf_Signed)entry->dw_offset);
+        }
         printf(" Value is expr val ");
         if (!entry->dw_block.bl_data) {
             printf("Compiler or libdwarf botch, "
@@ -821,6 +987,7 @@ print_one_regentry(const char *prefix,
         }
         /* printf(" block-ptr  0x%" DW_PR_DUx " ",
             (Dwarf_Unsigned)entry->dw_block.bl_data); */
+        printf("]");
         break;
     default: break;
     }
@@ -837,7 +1004,6 @@ print_regtable(Dwarf_Regtable3 *tab3)
         max = tab3->rt3_reg_table_size;
     }
     print_one_regentry("cfa",&tab3->rt3_cfa_rule);
-
     for (r = 0; r < max; r++) {
         char rn[30];
         snprintf(rn,sizeof(rn),"reg %d",r);
