@@ -1,35 +1,9 @@
 /*
-  Copyright (c) 2009-2020 David Anderson.  All rights reserved.
+  Copyright (c) 2009-2023 David Anderson.  All rights reserved.
 
-Redistribution and use in source and binary forms, with
-or without modification, are permitted provided that the
-following conditions are met:
-
-    (1) Redistributions of source code must retain the above
-    copyright notice, this list of conditions and the
-    following disclaimer.
-
-    (2) Redistributions in binary form must reproduce the
-    above copyright notice, this list of conditions and
-    the following disclaimer in the documentation and/or
-    other materials provided with the distribution.
-
-    (3)The name of the author may not be used to endorse
-    or promote products derived from this software without
-    specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
-EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
-THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
-NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
-EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+  This example program is hereby
+  placed into the public domain to be copied or
+  used by anyone for any purpose without restriction.
 
 */
 /*  simplereader.c
@@ -96,32 +70,28 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
         make
         ./simplereader simplereader
 */
-#include "config.h"
 
-/* Windows specific header files */
-#if defined(_WIN32) && defined(HAVE_STDAFX_H)
-#include "stdafx.h"
-#endif /* HAVE_STDAFX_H */
+#include <config.h>
+
+#include <stdio.h>  /* fprintf() printf() snprintf() */
+#include <stdlib.h> /* atoi() exit() free() */
+#include <string.h> /* memset() strcmp() strcpy() strdup()
+    strlen() strncmp() */
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h> /* For open() */
-#endif /* HAVE_SYS_TYPES_H */
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>  /* For open() */
-#endif /* HAVE_SYS_STAT_H */
-#include <fcntl.h>     /* For open() */
-#include <stdlib.h> /* for exit() */
-#ifdef _WIN32
-#include <io.h> /* for close() */
-#elif defined HAVE_UNISTD_H
-#include <unistd.h> /* for close() */
-#endif /* _WIN32 */
-#include <stdio.h>
-#include <errno.h>
-#include <string.h>
 #ifdef HAVE_STDINT_H
-#include <stdint.h> /* For uintptr_t */
+#include <stdint.h> /* uintptr_t */
 #endif /* HAVE_STDINT_H */
+
+#ifdef _WIN32
+#include <io.h> /* close() open() */
+#elif defined HAVE_UNISTD_H
+#include <unistd.h> /* close() */
+#endif /* _WIN32 */
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h> /* open() O_RDONLY */
+#endif /* HAVE_FCNTL_H */
+
 #include "dwarf.h"
 #include "libdwarf.h"
 #include "libdwarf_private.h"
@@ -146,9 +116,6 @@ struct srcfilesdata {
     int srcfilesres;
 };
 
-#define TRUE 1
-#define FALSE 0
-
 static void read_cu_list(Dwarf_Debug dbg);
 static void print_die_data(Dwarf_Debug dbg, Dwarf_Die print_me,
     int level,
@@ -209,7 +176,7 @@ cleanupstr(void)
     dupstrused = 0;
 }
 
-static unsigned  char_to_uns4bit(unsigned char c)
+static unsigned  char_to_uns4bit(Dwarf_Debug dbg,unsigned char c)
 {
     unsigned v;
     if ( c >= '0' && c <= '9') {
@@ -221,14 +188,18 @@ static unsigned  char_to_uns4bit(unsigned char c)
     else if ( c >= 'A' && c <= 'F') {
         v =  c - 'A' + 10;
     } else {
-        printf("Garbage hex char in %c 0x%x\n",c,c);
-        exit(1);
+        printf("ERROR Garbage hex char in %c 0x%x\n",c,c);
+        cleanupstr();
+        dwarf_finish(dbg);
+        exit(EXIT_FAILURE);
     }
     return v;
 }
 
 static void
-xfrm_to_sig8(const char *cuhash_in, Dwarf_Sig8 *hash_out)
+xfrm_to_sig8(Dwarf_Debug dbg,
+    const char *cuhash_in,
+    Dwarf_Sig8 *hash_out)
 {
     char localhash[16];
     unsigned hashin_len = strlen(cuhash_in);
@@ -240,7 +211,9 @@ xfrm_to_sig8(const char *cuhash_in, Dwarf_Sig8 *hash_out)
     if (hashin_len > fixed_size) {
         printf("FAIL: argument hash too long, len %u val:\"%s\"\n",
             hashin_len, cuhash_in);
-        exit(1);
+        dwarf_finish(dbg);
+        cleanupstr();
+        exit(EXIT_FAILURE);
     }
     if (hashin_len  < fixed_size) {
         unsigned add_zeros = fixed_size - hashin_len;
@@ -261,8 +234,8 @@ xfrm_to_sig8(const char *cuhash_in, Dwarf_Sig8 *hash_out)
         unsigned char hichar = localhash[2*i];
         unsigned char lochar = localhash[2*i+1];
         hash_out->signature[i] =
-            (char_to_uns4bit(hichar) << 4)  |
-            char_to_uns4bit(lochar);
+            (char_to_uns4bit(dbg,hichar) << 4)  |
+            char_to_uns4bit(dbg,lochar);
     }
     printf("Hex key = 0x");
     for (i = 0; i < sizeof(Dwarf_Sig8) ; ++i) {
@@ -305,7 +278,8 @@ startswithextractstring(const char *arg,const char *lookfor,
     if (dupstrused >= DUPSTRARRAYSIZE) {
         printf("FAIL: increase the value DUPSTRARRAYSIZE"
             " for test purposes\n");
-        exit(1);
+        cleanupstr();
+        exit(EXIT_FAILURE);
     }
     return TRUE;
 }
@@ -316,16 +290,19 @@ format_sig8_string(Dwarf_Sig8*data, char* str_buf,unsigned
 {
     unsigned i = 0;
     char *cp = str_buf;
+    char *sigbytep = &data->signature[0];
+
     if (buf_size <  19) {
         printf("FAIL: internal coding error in test.\n");
-        exit(1);
+        cleanupstr();
+        exit(EXIT_FAILURE);
     }
     strcpy(cp,"0x");
     cp += 2;
     buf_size -= 2;
-    for (; i < sizeof(data->signature); ++i,cp+=2,buf_size--) {
-        snprintf(cp, buf_size, "%02x",
-            (unsigned char)(data->signature[i]));
+    for (; i < sizeof(Dwarf_Sig8); ++i,cp+=2,buf_size-=2) {
+        snprintf(cp, buf_size, "%02x", *(unsigned char *)sigbytep);
+        ++sigbytep;
     }
     return;
 }
@@ -381,9 +358,11 @@ print_debug_fission_header(struct Dwarf_Debug_Fission_Per_CU_s *fsd)
     test how well that action works.   */
 static void
 simple_error_handler(Dwarf_Error error,
-    Dwarf_Ptr errarg UNUSEDARG)
+    Dwarf_Ptr errarg)
 {
     Dwarf_Unsigned earg =  (Dwarf_Unsigned)(uintptr_t)errarg;
+
+    (void)errarg;
     printf("\nlibdwarf error detected: 0x%" DW_PR_DUx " %s\n",
         dwarf_errno(error),dwarf_errmsg(error));
     printf("libdwarf errarg. %" DW_PR_DUu "\n", earg);
@@ -407,56 +386,88 @@ main(int argc, char **argv)
     int i = 0;
     #define MACHO_PATH_LEN 2000
     char macho_real_path[MACHO_PATH_LEN];
+    int dup_attr_check = TRUE;
 
     macho_real_path[0] = 0;
     for (i = 1; i < (argc-1) ; ++i) {
         if (strcmp(argv[i],"--names") == 0) {
             namesoptionon=1;
-        } else if (startswithextractstring(argv[1],"--dumpallnames=",
+            continue;
+        }
+        if (startswithextractstring(argv[1],"--dumpallnames=",
             &dumpallnamespath)) {
             dumpallnames=1;
-        } else if (strcmp(argv[i],"--check") == 0) {
+            continue;
+        }
+        if (strcmp(argv[i],"--check") == 0) {
             checkoptionon=1;
-        } else if (startswithextractstring(argv[i],
+            continue;
+        }
+        if (startswithextractstring(argv[i],
             "--tuhash=",&tuhash)) {
-            /* done */
-        } else if (startswithextractstring(argv[i],
+            continue;
+        }
+        if (startswithextractstring(argv[i],
             "--cuhash=",&cuhash)) {
-            /* done */
-        } else if (startswithextractstring(argv[i],
-            "--tufissionhash=",
-            &tufissionhash)) {
-            /* done */
-        } else if (startswithextractstring(argv[i],
+            continue;
+        }
+        if (startswithextractstring(argv[i],
+            "--tufissionhash=", &tufissionhash)) {
+            continue;
+        }
+        if (startswithextractstring(argv[i],
             "--cufissionhash=",
             &cufissionhash)) {
-            /* done */
-        } else if (strcmp(argv[i],"--passnullerror") == 0) {
+            continue;
+        }
+        if (strcmp(argv[i],"--passnullerror") == 0) {
             passnullerror=1;
-        } else if (strcmp(argv[i],"--simpleerrhand") == 0) {
+            /*  If null error, we cannot do much.
+                Do not expect to avoid a like without
+                help from libdwarf. */
+            dwarf_set_de_alloc_flag(TRUE);
+            continue;
+        }
+        if (strcmp(argv[i],"--simpleerrhand") == 0) {
             simpleerrhand=1;
-        } else if (startswithextractnum(argv[i],
+            continue;
+        }
+        if (strcmp(argv[i],"--no-dup-attr-check") == 0) {
+            dup_attr_check = FALSE;
+            continue;
+        }
+        if (startswithextractnum(argv[i],
             "--isinfo=",&g_is_info)) {
-            /* done */
-        } else if (startswithextractnum(argv[i],
+            continue;
+        }
+        if (startswithextractnum(argv[i],
             "--type=",&unittype)) {
-            /* done */
-        } else if (startswithextractnum(argv[i],
+            continue;
+        }
+        if (startswithextractnum(argv[i],
             "--fissionfordie=",
             &fissionfordie)) {
-            /* done */
-        } else if (!strcmp(argv[i],"--use-init-fd")) {
+            continue;
+        }
+        if (!strcmp(argv[i],"--use-init-fd")) {
             use_init_fd = TRUE;
             /* done */
-        } else {
-            printf("Unknown argument \"%s\", give up \n",argv[i]);
-            exit(1);
+        }
+        if (!strcmp(argv[i],"--suppress-de-alloc-tree")) {
+            /*  Never allow the call to suppress
+                alloc tracking in libdwarf as without normal
+                alloc tracking this source will have
+                leaks. Especially on corrupt DWARF or
+                corrupt Elf.
+                Just accept the option and continue.  */
+            continue;
         }
     }
     if (i >= argc) {
         printf("simplereader not given file to open\n");
         printf("simplereader exits\n");
-        exit(1);
+        cleanupstr();
+        exit(EXIT_FAILURE);
     }
     filepath = argv[i];
     if (dumpallnames) {
@@ -465,13 +476,15 @@ main(int argc, char **argv)
                 "(%s) "
                 "as the file to read is not allowed. giving up.\n",
                 filepath);
-            exit(1);
+            cleanupstr();
+            exit(EXIT_FAILURE);
         }
         dumpallnamesfile = fopen(dumpallnamespath,"w");
         if (!dumpallnamesfile) {
             printf("Cannot open %s. Giving up.\n",
                 dumpallnamespath);
-            exit(1);
+            cleanupstr();
+            exit(EXIT_FAILURE);
         }
     }
     if (passnullerror) {
@@ -484,6 +497,9 @@ main(int argc, char **argv)
         /* Not a very useful errarg... */
         errarg = (Dwarf_Ptr)1;
     }
+    if (!dup_attr_check) {
+        dwarf_library_allow_dup_attr(TRUE);
+    }
     if (use_init_fd) {
         /*  For testing a libdwarf init function.
             We are not finding the true dSYM Macho-object
@@ -496,7 +512,8 @@ main(int argc, char **argv)
         my_init_fd = open(filepath,O_RDONLY|O_BINARY);
         if (my_init_fd == -1) {
             printf("Giving up, cannot open %s\n",filepath);
-            exit(1);
+            cleanupstr();
+            exit(EXIT_FAILURE);
         }
         res = dwarf_init_b(my_init_fd,
             DW_GROUPNUMBER_ANY,
@@ -508,16 +525,23 @@ main(int argc, char **argv)
             DW_GROUPNUMBER_ANY,errhand,errarg,&dbg, errp);
     }
     if (res != DW_DLV_OK) {
+        if (res == DW_DLV_ERROR) {
+            if (errp) {
+                dwarf_dealloc_error(dbg,*errp);
+            }
+            error = 0;
+        }
         printf("Giving up, cannot do DWARF processing %s\n",
             filepath?filepath:"");
         cleanupstr();
-        exit(1);
+        dwarf_finish(dbg);
+        exit(EXIT_FAILURE);
     }
 
     if (cuhash) {
         Dwarf_Die die;
         stdrun = FALSE;
-        xfrm_to_sig8(cuhash,&hash8);
+        xfrm_to_sig8(dbg,cuhash,&hash8);
         printf("\n");
         printf("Getting die for CU key %s\n",cuhash);
         res = dwarf_die_from_hash_signature(dbg,
@@ -541,7 +565,7 @@ main(int argc, char **argv)
     if (tuhash) {
         Dwarf_Die die;
         stdrun = FALSE;
-        xfrm_to_sig8(tuhash,&hash8);
+        xfrm_to_sig8(dbg,tuhash,&hash8);
         printf("\n");
         printf("Getting die for TU key %s\n",tuhash);
         res = dwarf_die_from_hash_signature(dbg,
@@ -566,7 +590,7 @@ main(int argc, char **argv)
         Dwarf_Debug_Fission_Per_CU  fisdata;
         stdrun = FALSE;
         memset(&fisdata,0,sizeof(fisdata));
-        xfrm_to_sig8(cufissionhash,&hash8);
+        xfrm_to_sig8(dbg,cufissionhash,&hash8);
         printf("\n");
         printf("Getting fission data for CU key %s\n",cufissionhash);
         res = dwarf_get_debugfission_for_key(dbg,
@@ -586,7 +610,7 @@ main(int argc, char **argv)
         Dwarf_Debug_Fission_Per_CU  fisdata;
         stdrun = FALSE;
         memset(&fisdata,0,sizeof(fisdata));
-        xfrm_to_sig8(tufissionhash,&hash8);
+        xfrm_to_sig8(dbg,tufissionhash,&hash8);
         printf("\n");
         printf("Getting fission data for TU key %s\n",tufissionhash);
         res = dwarf_get_debugfission_for_key(dbg,
@@ -634,10 +658,9 @@ read_cu_list(Dwarf_Debug dbg)
     Dwarf_Half     header_cu_type = unittype;
     Dwarf_Bool     is_info = g_is_info;
     Dwarf_Error error;
-    int cu_number = 0;
     Dwarf_Error *errp  = 0;
 
-    for (;;++cu_number) {
+    for (;;) {
         Dwarf_Die no_die = 0;
         Dwarf_Die cu_die = 0;
         int res = DW_DLV_ERROR;
@@ -659,10 +682,15 @@ read_cu_list(Dwarf_Debug dbg)
             &typeoffset, &next_cu_header,
             &header_cu_type,errp);
         if (res == DW_DLV_ERROR) {
-            char *em = errp?dwarf_errmsg(error):
+            char *em = errp?dwarf_errmsg(*errp):
                 "An error next cu her";
             printf("Error in dwarf_next_cu_header: %s\n",em);
-            exit(1);
+            if (errp) {
+                dwarf_dealloc_error(dbg,*errp);
+            }
+            dwarf_finish(dbg);
+            cleanupstr();
+            exit(EXIT_FAILURE);
         }
         if (res == DW_DLV_NO_ENTRY) {
             /* Done. */
@@ -674,17 +702,24 @@ read_cu_list(Dwarf_Debug dbg)
         res = dwarf_siblingof_b(dbg,no_die,is_info,
             &cu_die,errp);
         if (res == DW_DLV_ERROR) {
-            char *em = errp?dwarf_errmsg(error):"An error";
+            char *em = (errp?dwarf_errmsg(*errp):"An error");
             printf("Error in dwarf_siblingof_b on CU die: %s\n",em);
-            exit(1);
+            if (errp) {
+                dwarf_dealloc_error(dbg,*errp);
+            }
+            dwarf_finish(dbg);
+            cleanupstr();
+            exit(EXIT_FAILURE);
         }
         if (res == DW_DLV_NO_ENTRY) {
             /* Impossible case. */
             printf("no entry! in dwarf_siblingof on CU die \n");
-            exit(1);
+            dwarf_finish(dbg);
+            cleanupstr();
+            exit(EXIT_FAILURE);
         }
         get_die_and_siblings(dbg,cu_die,is_info,0,&sf);
-        dwarf_dealloc(dbg,cu_die,DW_DLA_DIE);
+        dwarf_dealloc_die(cu_die);
         resetsrcfiles(dbg,&sf);
     }
 }
@@ -712,13 +747,19 @@ get_die_and_siblings(Dwarf_Debug dbg, Dwarf_Die in_die,
         res = dwarf_child(cur_die,&child,errp);
         if (res == DW_DLV_ERROR) {
             printf("Error in dwarf_child , level %d \n",in_level);
-            exit(1);
+            dwarf_dealloc_die(cur_die);
+            if (errp) {
+                dwarf_dealloc_error(dbg,*errp);
+            }
+            dwarf_finish(dbg);
+            cleanupstr();
+            exit(EXIT_FAILURE);
         }
         if (res == DW_DLV_OK) {
             get_die_and_siblings(dbg,child,is_info,
                 in_level+1,sf);
             /* No longer need 'child' die. */
-            dwarf_dealloc(dbg,child,DW_DLA_DIE);
+            dwarf_dealloc_die(child);
             child = 0;
         }
         /* res == DW_DLV_NO_ENTRY or DW_DLV_OK */
@@ -727,7 +768,9 @@ get_die_and_siblings(Dwarf_Debug dbg, Dwarf_Die in_die,
             char *em = errp?dwarf_errmsg(error):"Error siblingof_b";
             printf("Error in dwarf_siblingof_b , level %d :%s \n",
                 in_level,em);
-            exit(1);
+            dwarf_finish(dbg);
+            cleanupstr();
+            exit(EXIT_FAILURE);
         }
         if (res == DW_DLV_NO_ENTRY) {
             /* Done at this level. */
@@ -735,8 +778,7 @@ get_die_and_siblings(Dwarf_Debug dbg, Dwarf_Die in_die,
         }
         /* res == DW_DLV_OK */
         if (cur_die != in_die) {
-            dwarf_dealloc(dbg,cur_die,DW_DLA_DIE);
-            cur_die = 0;
+            dwarf_dealloc_die(cur_die);
         }
         cur_die = sib_die;
         print_die_data(dbg,cur_die,in_level,sf);
@@ -816,10 +858,12 @@ print_subprog(Dwarf_Debug dbg,Dwarf_Die die,
     res = dwarf_attrlist(die,&attrbuf,&attrcount,errp);
     if (res != DW_DLV_OK) {
         if (res == DW_DLV_ERROR) {
-            if (!passnullerror) {
+            if (errp) {
                 dwarf_dealloc_error(dbg,error);
                 error = 0;
-                exit(1);
+                dwarf_finish(dbg);
+                cleanupstr();
+                exit(EXIT_FAILURE);
             }
         }
         return;
@@ -852,11 +896,11 @@ print_subprog(Dwarf_Debug dbg,Dwarf_Die die,
                 get_addr(attrbuf[i],&highpc);
             }
         }
-        if (res == DW_DLV_ERROR && !passnullerror) {
+        if (res == DW_DLV_ERROR && errp) {
             dwarf_dealloc_error(dbg,error);
             error = 0;
         }
-        dwarf_dealloc(dbg,attrbuf[i],DW_DLA_ATTR);
+        dwarf_dealloc_attribute(attrbuf[i]);
     }
     /*  Here let's test some alternative interfaces for
         high and low pc. */
@@ -1015,14 +1059,24 @@ print_name_strings_attr(Dwarf_Debug dbg, Dwarf_Die die,
 
     res = dwarf_whatattr(attr,&attrnum,&error);
     if (res != DW_DLV_OK) {
+        if (res == DW_DLV_ERROR) {
+            dwarf_dealloc_error(dbg,error);
+        }
         printf("Unable to get attr number");
-        exit(1);
+        dwarf_finish(dbg);
+        cleanupstr();
+        exit(EXIT_FAILURE);
     }
 
     res = dwarf_whatform(attr,&finalform,&error);
     if (res != DW_DLV_OK) {
+        if (res == DW_DLV_ERROR) {
+            dwarf_dealloc_error(dbg,error);
+        }
         printf("Unable to get attr form");
-        exit(1);
+        dwarf_finish(dbg);
+        cleanupstr();
+        exit(EXIT_FAILURE);
     }
 
     cl = dwarf_get_form_class(cu_version_stamp,
@@ -1045,6 +1099,9 @@ printnamestrings(Dwarf_Debug dbg, Dwarf_Die die)
 
     res = dwarf_attrlist(die,&atlist, &atcount,&error);
     if (res != DW_DLV_OK) {
+        if (res == DW_DLV_ERROR) {
+            dwarf_dealloc_error(dbg,error);
+        }
         return;
     }
     for (i = 0; i < atcount; ++i) {
@@ -1054,7 +1111,6 @@ printnamestrings(Dwarf_Debug dbg, Dwarf_Die die)
         print_name_strings_attr(dbg,die,attr);
     }
     dwarf_dealloc(dbg,atlist, DW_DLA_LIST);
-
 }
 
 static void
@@ -1080,7 +1136,9 @@ print_die_data_i(Dwarf_Debug dbg, Dwarf_Die print_me,
     res = dwarf_diename(print_me,&name,errp);
     if (res == DW_DLV_ERROR) {
         printf("Error in dwarf_diename , level %d \n",level);
-        exit(1);
+        dwarf_finish(dbg);
+        cleanupstr();
+        exit(EXIT_FAILURE);
     }
     if (res == DW_DLV_NO_ENTRY) {
         name = "<no DW_AT_name attr>";
@@ -1088,12 +1146,22 @@ print_die_data_i(Dwarf_Debug dbg, Dwarf_Die print_me,
     res = dwarf_tag(print_me,&tag,errp);
     if (res != DW_DLV_OK) {
         printf("Error in dwarf_tag , level %d \n",level);
-        exit(1);
+        if (res == DW_DLV_ERROR && errp) {
+            dwarf_dealloc_error(dbg,*errp);
+        }
+        cleanupstr();
+        dwarf_finish(dbg);
+        exit(EXIT_FAILURE);
     }
     res = dwarf_get_TAG_name(tag,&tagname);
     if (res != DW_DLV_OK) {
+        if (res == DW_DLV_ERROR && errp) {
+            dwarf_dealloc_error(dbg,*errp);
+        }
         printf("Error in dwarf_get_TAG_name , level %d \n",level);
-        exit(1);
+        cleanupstr();
+        dwarf_finish(dbg);
+        exit(EXIT_FAILURE);
     }
     if (dumpallnames) {
         printnamestrings(dbg,print_me);
@@ -1104,8 +1172,13 @@ print_die_data_i(Dwarf_Debug dbg, Dwarf_Die print_me,
     } else {
         res = dwarf_whatform(attr,&formnum,errp);
         if (res != DW_DLV_OK) {
+            if (res == DW_DLV_ERROR && errp) {
+                dwarf_dealloc_error(dbg,*errp);
+            }
             printf("Error in dwarf_whatform , level %d \n",level);
-            exit(1);
+            dwarf_finish(dbg);
+            cleanupstr();
+            exit(EXIT_FAILURE);
         }
         formname = "form-name-unavailable";
         res = dwarf_get_FORM_name(formnum,&formname);
@@ -1172,16 +1245,25 @@ print_die_data(Dwarf_Debug dbg, Dwarf_Die print_me,
                 printf("FAIL: Error in "
                     "dwarf_get_debugfission_for_die %d\n",
                     fissionfordie);
-                exit(1);
+                if (errp) {
+                    dwarf_dealloc_error(dbg,*errp);
+                }
+                dwarf_finish(dbg);
+                cleanupstr();
+                exit(EXIT_FAILURE);
             }
             if (res == DW_DLV_NO_ENTRY) {
                 printf("FAIL: no-entry in "
                     "dwarf_get_debugfission_for_die %d\n",
                     fissionfordie);
-                exit(1);
+                dwarf_finish(dbg);
+                cleanupstr();
+                exit(EXIT_FAILURE);
             }
             print_die_data_i(dbg,print_me,level,sf);
             print_debug_fission_header(&percu);
+            dwarf_finish(dbg);
+            cleanupstr();
             exit(0);
         }
         dienumber++;
