@@ -44,6 +44,7 @@ Index-Section-Format.html#Index-Section-Format
 
 #include "dwarf.h"
 #include "libdwarf.h"
+#include "dwarf_local_malloc.h"
 #include "libdwarf_private.h"
 #include "dwarf_base_types.h"
 #include "dwarf_opaque.h"
@@ -63,7 +64,7 @@ Index-Section-Format.html#Index-Section-Format
 #if WORDS_BIGENDIAN   /* meaning on this host */
 #define READ_GDBINDEX(dest,desttype, source, length) \
     do {                                             \
-        BIGGEST_UINT _ltmp = 0;                      \
+        desttype _ltmp = 0;                      \
         _dwarf_memcpy_swap_bytes((((char *)(&_ltmp)) \
             + sizeof(_ltmp) - (length)),             \
             (source), (length)) ;                    \
@@ -72,22 +73,45 @@ Index-Section-Format.html#Index-Section-Format
 #else /* little-endian on this host */
 #define READ_GDBINDEX(dest,desttype, source, length) \
     do {                                             \
-        BIGGEST_UINT _ltmp = 0;                      \
+        desttype _ltmp = 0;                      \
         memcpy(((char *)(&_ltmp)) ,                  \
             (source), (length)) ;                    \
         (dest) = (desttype)_ltmp;                    \
     } while (0)
 #endif
 
-struct gi_fileheader_s {
-    char gfs [4][6];
-};
-
 struct dwarf_64bitpair {
     gdbindex_64 offset;
     gdbindex_64 length;
 };
 
+static void
+emit_no_value_msg(Dwarf_Debug dbg,
+    int errnum,
+    const char * errstr_text,
+    Dwarf_Error *error)
+{
+    _dwarf_error_string(dbg,error,errnum,
+        (char *)errstr_text);
+}
+
+static void
+emit_one_value_msg(Dwarf_Debug dbg,
+    int errnum,
+    const char * errstr_text,
+    Dwarf_Unsigned value,
+    Dwarf_Error *error)
+{
+    dwarfstring m;
+
+    dwarfstring_constructor(&m);
+    dwarfstring_append_printf_u(&m,
+        (char *)errstr_text,value);
+    _dwarf_error_string(dbg,error,errnum,
+        dwarfstring_string(&m));
+    dwarfstring_destructor(&m);
+}
+
 static int
 set_base(Dwarf_Debug dbg,
     struct Dwarf_Gdbindex_array_instance_s * hdr,
@@ -99,7 +123,7 @@ set_base(Dwarf_Debug dbg,
     /* The size of each field in the struct in the object. */
     Dwarf_Unsigned fieldlen,
     enum gdbindex_type_e type,
-    Dwarf_Error * err)
+    Dwarf_Error * error)
 {
 
     if (type == git_std || type == git_cuvec) {
@@ -107,7 +131,7 @@ set_base(Dwarf_Debug dbg,
             section, but a useful one. */
         Dwarf_Unsigned count = 0;
         if ( end < start) {
-            _dwarf_error(dbg, err,DW_DLE_GDB_INDEX_COUNT_ERROR);
+            _dwarf_error(dbg, error,DW_DLE_GDB_INDEX_COUNT_ERROR);
             return DW_DLV_ERROR;
         }
         count = end - start;
@@ -116,14 +140,15 @@ set_base(Dwarf_Debug dbg,
         hdr->dg_base = start;
         hdr->dg_count = count;
         hdr->dg_entry_length = entrylen;
-        hdr->dg_fieldlen = fieldlen;
+        hdr->dg_fieldlen = (unsigned)fieldlen;
     } else {
         /* address area. */
         /* 64bit, 64bit, offset. Then 32bit pad. */
         Dwarf_Unsigned count = 0;
         hdr->dg_base = start;
         if ( end < start) {
-            _dwarf_error(dbg, err,DW_DLE_GDB_INDEX_COUNT_ADDR_ERROR);
+            _dwarf_error(dbg, error,
+                DW_DLE_GDB_INDEX_COUNT_ADDR_ERROR);
             return DW_DLV_ERROR;
         }
         /* entry length includes pad. */
@@ -153,11 +178,13 @@ dwarf_gdbindex_header(Dwarf_Debug dbg,
     const char    ** section_name,
     Dwarf_Error    * error)
 {
-
-    struct gi_fileheader_s header;
     Dwarf_Gdbindex indexptr = 0;
     int res = DW_DLV_ERROR;
+    Dwarf_Small *data = 0;
+    Dwarf_Small *startdata = 0;
+    Dwarf_Unsigned version_in = 0;
 
+    CHECK_DBG(dbg,error,"dwarf_gdbindex_header()");
     if (!dbg->de_debug_gdbindex.dss_size) {
         return DW_DLV_NO_ENTRY;
     }
@@ -167,14 +194,24 @@ dwarf_gdbindex_header(Dwarf_Debug dbg,
             return res;
         }
     }
+    data = dbg->de_debug_gdbindex.dss_data;
+    if (!data) {
+        /*  Should be impossible, dwarf_load_section() would
+            return DW_DLV_ERROR if dss_data could not be
+            set non-null */
+        _dwarf_error_string(dbg, error,
+            DW_DLE_ERRONEOUS_GDB_INDEX_SECTION,
+            "DW_DLE_ERRONEOUS_GDB_INDEX_SECTION: "
+            "We have non-zero (section) dss_size but "
+            "null dss_data pointer");
+        return DW_DLV_ERROR;
+    }
+    startdata = data;
 
-    if (dbg->de_debug_gdbindex.dss_size <
-        sizeof(struct gi_fileheader_s) ) {
+    if (dbg->de_debug_gdbindex.dss_size < (DWARF_32BIT_SIZE*6)) {
         _dwarf_error(dbg, error, DW_DLE_ERRONEOUS_GDB_INDEX_SECTION);
         return DW_DLV_ERROR;
     }
-    memcpy(&header,dbg->de_debug_gdbindex.dss_data,
-        sizeof(struct gi_fileheader_s));
     indexptr = (Dwarf_Gdbindex)_dwarf_get_alloc(dbg,
         DW_DLA_GDBINDEX,1);
     if (indexptr == NULL) {
@@ -182,34 +219,92 @@ dwarf_gdbindex_header(Dwarf_Debug dbg,
             "DW_DLE_ALLOC_FAIL: allocating Dwarf_Gdbindex");
         return DW_DLV_ERROR;
     }
+    READ_GDBINDEX(version_in,Dwarf_Unsigned,
+        data, DWARF_32BIT_SIZE);
+    indexptr->gi_version = version_in;
 
     indexptr->gi_dbg = dbg;
-    indexptr->gi_section_data = dbg->de_debug_gdbindex.dss_data;
+    indexptr->gi_section_data = startdata;
     indexptr->gi_section_length = dbg->de_debug_gdbindex.dss_size;
-    READ_GDBINDEX(indexptr->gi_version ,Dwarf_Unsigned,
-        dbg->de_debug_gdbindex.dss_data,
-        DWARF_32BIT_SIZE);
+    /*   7 and lower are different format in some way */
+    if (indexptr->gi_version != 8 &&
+        indexptr->gi_version != 7) {
+        emit_one_value_msg(dbg, DW_DLE_ERRONEOUS_GDB_INDEX_SECTION,
+            "DW_DLE_ERRONEOUS_GDB_INDEX_SECTION: "
+            " version number %u is not"
+            " supported",
+            indexptr->gi_version,error);
+        dwarf_dealloc(dbg,indexptr,DW_DLA_GDBINDEX);
+        return DW_DLV_ERROR;
+    }
+    data += DWARF_32BIT_SIZE;
     READ_GDBINDEX(indexptr->gi_cu_list_offset ,Dwarf_Unsigned,
-        dbg->de_debug_gdbindex.dss_data + DWARF_32BIT_SIZE,
-        DWARF_32BIT_SIZE);
+        data, DWARF_32BIT_SIZE);
+    if (indexptr->gi_cu_list_offset > indexptr->gi_section_length) {
+        emit_one_value_msg(dbg, DW_DLE_ERRONEOUS_GDB_INDEX_SECTION,
+            "DW_DLE_ERRONEOUS_GDB_INDEX_SECTION"
+            " cu list offset of %u is too large for the section",
+            indexptr->gi_cu_list_offset,error);
+        dwarf_dealloc(dbg,indexptr,DW_DLA_GDBINDEX);
+        return DW_DLV_ERROR;
+    }
+    data += DWARF_32BIT_SIZE;
     READ_GDBINDEX(indexptr->gi_types_cu_list_offset ,Dwarf_Unsigned,
-        dbg->de_debug_gdbindex.dss_data + 2*DWARF_32BIT_SIZE,
-        DWARF_32BIT_SIZE);
+        data, DWARF_32BIT_SIZE);
+    if (indexptr->gi_types_cu_list_offset >
+        indexptr->gi_section_length) {
+        emit_one_value_msg(dbg, DW_DLE_ERRONEOUS_GDB_INDEX_SECTION,
+            "DW_DLE_ERRONEOUS_GDB_INDEX_SECTION"
+            " types cu list offset of %u is too "
+            "large for the section",
+            indexptr->gi_cu_list_offset,error);
+        dwarf_dealloc(dbg,indexptr,DW_DLA_GDBINDEX);
+        return DW_DLV_ERROR;
+    }
+    data += DWARF_32BIT_SIZE;
     READ_GDBINDEX(indexptr->gi_address_area_offset ,Dwarf_Unsigned,
-        dbg->de_debug_gdbindex.dss_data + 3*DWARF_32BIT_SIZE,
-        DWARF_32BIT_SIZE);
+        data, DWARF_32BIT_SIZE);
+    if (indexptr->gi_address_area_offset >
+        indexptr->gi_section_length) {
+        emit_one_value_msg(dbg, DW_DLE_ERRONEOUS_GDB_INDEX_SECTION,
+            "DW_DLE_ERRONEOUS_GDB_INDEX_SECTION"
+            " address area offset of %u is too "
+            "large for the section",
+            indexptr->gi_address_area_offset ,error);
+        dwarf_dealloc(dbg,indexptr,DW_DLA_GDBINDEX);
+        return DW_DLV_ERROR;
+    }
+    data += DWARF_32BIT_SIZE;
     READ_GDBINDEX(indexptr->gi_symbol_table_offset ,Dwarf_Unsigned,
-        dbg->de_debug_gdbindex.dss_data + 4*DWARF_32BIT_SIZE,
-        DWARF_32BIT_SIZE);
+        data, DWARF_32BIT_SIZE);
+    if (indexptr->gi_symbol_table_offset >
+        indexptr->gi_section_length) {
+        emit_one_value_msg(dbg, DW_DLE_ERRONEOUS_GDB_INDEX_SECTION,
+            "DW_DLE_ERRONEOUS_GDB_INDEX_SECTION"
+            " symbol table offset of %u is too "
+            "large for the section",
+            indexptr->gi_symbol_table_offset,error);
+        dwarf_dealloc(dbg,indexptr,DW_DLA_GDBINDEX);
+        return DW_DLV_ERROR;
+    }
+    data += DWARF_32BIT_SIZE;
     READ_GDBINDEX(indexptr->gi_constant_pool_offset ,Dwarf_Unsigned,
-        dbg->de_debug_gdbindex.dss_data + 5*DWARF_32BIT_SIZE,
-        DWARF_32BIT_SIZE);
+        data, DWARF_32BIT_SIZE);
+    if (indexptr->gi_constant_pool_offset >
+        indexptr->gi_section_length) {
+        emit_one_value_msg(dbg, DW_DLE_ERRONEOUS_GDB_INDEX_SECTION,
+            "DW_DLE_ERRONEOUS_GDB_INDEX_SECTION"
+            " constant pool offset of %u is too "
+            "large for the section",
+            indexptr->gi_constant_pool_offset,error);
+        dwarf_dealloc(dbg,indexptr,DW_DLA_GDBINDEX);
+        return DW_DLV_ERROR;
+    }
+    data += DWARF_32BIT_SIZE;
 
     res = set_base(dbg,&indexptr->gi_culisthdr,
-        dbg->de_debug_gdbindex.dss_data +
-        indexptr->gi_cu_list_offset,
-        dbg->de_debug_gdbindex.dss_data +
-        indexptr->gi_types_cu_list_offset,
+        startdata + indexptr->gi_cu_list_offset,
+        startdata + indexptr->gi_types_cu_list_offset,
         2*sizeof(gdbindex_64),
         sizeof(gdbindex_64),
         git_std,error);
@@ -218,10 +313,8 @@ dwarf_gdbindex_header(Dwarf_Debug dbg,
         return res;
     }
     res = set_base(dbg,&indexptr->gi_typesculisthdr,
-        dbg->de_debug_gdbindex.dss_data +
-        indexptr->gi_types_cu_list_offset,
-        dbg->de_debug_gdbindex.dss_data +
-        indexptr->gi_address_area_offset,
+        startdata+ indexptr->gi_types_cu_list_offset,
+        startdata+ indexptr->gi_address_area_offset,
         3*sizeof(gdbindex_64),
         sizeof(gdbindex_64),
         git_std,error);
@@ -230,10 +323,8 @@ dwarf_gdbindex_header(Dwarf_Debug dbg,
         return res;
     }
     res = set_base(dbg,&indexptr->gi_addressareahdr,
-        dbg->de_debug_gdbindex.dss_data +
-        indexptr->gi_address_area_offset,
-        dbg->de_debug_gdbindex.dss_data +
-        indexptr->gi_symbol_table_offset,
+        startdata + indexptr->gi_address_area_offset,
+        startdata + indexptr->gi_symbol_table_offset,
         3*sizeof(gdbindex_64),
         sizeof(gdbindex_64),
         git_address,error);
@@ -242,10 +333,8 @@ dwarf_gdbindex_header(Dwarf_Debug dbg,
         return res;
     }
     res = set_base(dbg,&indexptr->gi_symboltablehdr,
-        dbg->de_debug_gdbindex.dss_data +
-        indexptr->gi_symbol_table_offset,
-        dbg->de_debug_gdbindex.dss_data +
-            indexptr->gi_constant_pool_offset,
+        startdata + indexptr->gi_symbol_table_offset,
+        startdata + indexptr->gi_constant_pool_offset,
         2*DWARF_32BIT_SIZE,
         DWARF_32BIT_SIZE,
         git_std,error);
@@ -254,11 +343,10 @@ dwarf_gdbindex_header(Dwarf_Debug dbg,
         return res;
     }
     res = set_base(dbg,&indexptr->gi_cuvectorhdr,
-        dbg->de_debug_gdbindex.dss_data +
-            indexptr->gi_constant_pool_offset,
+        startdata + indexptr->gi_constant_pool_offset,
         /*  There is no real single vector size.
             but we'll use the entire rest as if there was. */
-        dbg->de_debug_gdbindex.dss_data + indexptr->gi_section_length,
+        startdata + indexptr->gi_section_length,
         DWARF_32BIT_SIZE,
         DWARF_32BIT_SIZE,
         git_cuvec,error);
@@ -266,10 +354,6 @@ dwarf_gdbindex_header(Dwarf_Debug dbg,
         dwarf_dealloc(dbg,indexptr,DW_DLA_GDBINDEX);
         return res;
     }
-
-    /* Really just pointing to constant pool area. */
-    indexptr->gi_string_pool = dbg->de_debug_gdbindex.dss_data +
-        indexptr->gi_constant_pool_offset;
     *gdbindexptr          = indexptr;
     *version              = indexptr->gi_version;
     *cu_list_offset       = indexptr->gi_cu_list_offset;
@@ -280,14 +364,22 @@ dwarf_gdbindex_header(Dwarf_Debug dbg,
     *section_size         = indexptr->gi_section_length;
     *section_name  =        dbg->de_debug_gdbindex.dss_name;
     return DW_DLV_OK;
-
 }
 
 int
 dwarf_gdbindex_culist_array(Dwarf_Gdbindex gdbindexptr,
     Dwarf_Unsigned       * list_length,
-    Dwarf_Error * error UNUSEDARG)
+    Dwarf_Error * error)
 {
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL inindexptr to"
+            " dwarf_gdbindex_culist_array");
+        return DW_DLV_ERROR;
+    }
+    (void)error;
     *list_length = gdbindexptr->gi_culisthdr.dg_count;
     return DW_DLV_OK;
 }
@@ -300,19 +392,42 @@ dwarf_gdbindex_culist_entry(Dwarf_Gdbindex gdbindexptr,
     Dwarf_Unsigned * cu_length,
     Dwarf_Error    * error)
 {
-    Dwarf_Unsigned max =  gdbindexptr->gi_culisthdr.dg_count;
-    Dwarf_Small * base = 0;
+    Dwarf_Small  * base = 0;
+    Dwarf_Small  * endptr = 0;
     Dwarf_Unsigned offset = 0;
     Dwarf_Unsigned length = 0;
-    unsigned fieldlen = gdbindexptr->gi_culisthdr.dg_fieldlen;
+    Dwarf_Unsigned max = 0;
+    unsigned       fieldlen = 0;
 
-    if (entryindex >= max) {
-        _dwarf_error(gdbindexptr->gi_dbg, error,
-            DW_DLE_GDB_INDEX_INDEX_ERROR);
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL inindexptr to"
+            " dwarf_gdbindex_culist_entry");
         return DW_DLV_ERROR;
     }
+    max =  gdbindexptr->gi_culisthdr.dg_count;
+    fieldlen = gdbindexptr->gi_culisthdr.dg_fieldlen;
+    if (entryindex >= max) {
+        return DW_DLV_NO_ENTRY;
+    }
+    endptr = gdbindexptr->gi_section_data +
+        gdbindexptr->gi_section_length;
+
     base = gdbindexptr->gi_culisthdr.dg_base;
     base += entryindex*gdbindexptr->gi_culisthdr.dg_entry_length;
+    if ((base + 2*fieldlen) >endptr) {
+        Dwarf_Debug    dbg = 0;
+
+        dbg = gdbindexptr->gi_dbg;
+        emit_one_value_msg(dbg, DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " end offset of data for index %u is past the"
+            " end of the section",
+            entryindex,error);
+        return DW_DLV_ERROR;
+    }
 
     READ_GDBINDEX(offset ,Dwarf_Unsigned,
         base,
@@ -328,8 +443,18 @@ dwarf_gdbindex_culist_entry(Dwarf_Gdbindex gdbindexptr,
 int
 dwarf_gdbindex_types_culist_array(Dwarf_Gdbindex gdbindexptr,
     Dwarf_Unsigned       * list_length,
-    Dwarf_Error  * error UNUSEDARG)
+    Dwarf_Error  * error)
 {
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL inindexptr to"
+            " dwarf_types_culist_entry");
+        return DW_DLV_ERROR;
+    }
+
+    (void)error;
     *list_length = gdbindexptr->gi_typesculisthdr.dg_count;
     return DW_DLV_OK;
 }
@@ -343,21 +468,41 @@ dwarf_gdbindex_types_culist_entry(Dwarf_Gdbindex gdbindexptr,
     Dwarf_Unsigned * t_signature,
     Dwarf_Error    * error)
 {
-    Dwarf_Unsigned max =  gdbindexptr->gi_typesculisthdr.dg_count;
+    Dwarf_Unsigned max =  0;
     Dwarf_Small * base = 0;
+    Dwarf_Small * endptr = 0;
     Dwarf_Unsigned offset = 0;
     Dwarf_Unsigned length = 0;
     Dwarf_Unsigned signature = 0;
-    unsigned fieldlen = gdbindexptr->gi_typesculisthdr.dg_fieldlen;
+    unsigned fieldlen = 0;
+
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL inindexptr to"
+            " dwarf_gdbindex_types_culist_entry");
+        return DW_DLV_ERROR;
+    }
+    fieldlen = gdbindexptr->gi_typesculisthdr.dg_fieldlen;
+    max =  gdbindexptr->gi_typesculisthdr.dg_count;
+    endptr = gdbindexptr->gi_section_data +
+        gdbindexptr->gi_section_length;
 
     if (entryindex >= max) {
-        _dwarf_error(gdbindexptr->gi_dbg, error,
-            DW_DLE_GDB_INDEX_INDEX_ERROR);
-        return DW_DLV_ERROR;
+        return DW_DLV_NO_ENTRY;
     }
     base = gdbindexptr->gi_typesculisthdr.dg_base;
     base += entryindex*gdbindexptr->gi_typesculisthdr.dg_entry_length;
-
+    if ((base + 3*fieldlen) >endptr) {
+        Dwarf_Debug dbg = gdbindexptr->gi_dbg;
+        emit_one_value_msg(dbg, DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " end offset of data for type index %u is past the"
+            " end of the section",
+            entryindex,error);
+        return DW_DLV_ERROR;
+    }
     READ_GDBINDEX(offset ,Dwarf_Unsigned,
         base,
         fieldlen);
@@ -376,8 +521,17 @@ dwarf_gdbindex_types_culist_entry(Dwarf_Gdbindex gdbindexptr,
 int
 dwarf_gdbindex_addressarea(Dwarf_Gdbindex gdbindexptr,
     Dwarf_Unsigned            * list_length,
-    Dwarf_Error     * error UNUSEDARG)
+    Dwarf_Error     * error)
 {
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL inindexptr to"
+            " dwarf_gdbindex_addressarea");
+        return DW_DLV_ERROR;
+    }
+    (void)error;
     *list_length = gdbindexptr->gi_addressareahdr.dg_count;
     return DW_DLV_OK;
 }
@@ -392,12 +546,23 @@ dwarf_gdbindex_addressarea_entry(
     Dwarf_Unsigned * cu_index,
     Dwarf_Error    * error)
 {
-    Dwarf_Unsigned max =  gdbindexptr->gi_addressareahdr.dg_count;
+    Dwarf_Unsigned max = 0;
     Dwarf_Small * base = 0;
+    Dwarf_Small * endptr = 0;
     Dwarf_Unsigned lowaddr = 0;
     Dwarf_Unsigned highaddr = 0;
     Dwarf_Unsigned cuindex = 0;
+    Dwarf_Unsigned fieldslen = 0;
 
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL inindexptr to"
+            " dwarf_gdbindex_addressarea_entry");
+        return DW_DLV_ERROR;
+    }
+    max =  gdbindexptr->gi_addressareahdr.dg_count;
     if (entryindex >= max) {
         _dwarf_error(gdbindexptr->gi_dbg, error,
             DW_DLE_GDB_INDEX_INDEX_ERROR);
@@ -405,6 +570,19 @@ dwarf_gdbindex_addressarea_entry(
     }
     base = gdbindexptr->gi_addressareahdr.dg_base;
     base += entryindex*gdbindexptr->gi_addressareahdr.dg_entry_length;
+    endptr = gdbindexptr->gi_section_data +
+        gdbindexptr->gi_section_length;
+    fieldslen = 2*sizeof(gdbindex_64) + DWARF_32BIT_SIZE;
+    if ((base + fieldslen) > endptr) {
+        Dwarf_Debug dbg = gdbindexptr->gi_dbg;
+        emit_one_value_msg(dbg, DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " end offset of data for "
+            " dwarf_gdbindex_addressarea_entry %u is past the"
+            " end of the section",
+            entryindex,error);
+        return DW_DLV_ERROR;
+    }
 
     READ_GDBINDEX(lowaddr ,Dwarf_Unsigned,
         base,
@@ -424,8 +602,17 @@ dwarf_gdbindex_addressarea_entry(
 int
 dwarf_gdbindex_symboltable_array(Dwarf_Gdbindex gdbindexptr,
     Dwarf_Unsigned            * list_length,
-    Dwarf_Error     * error UNUSEDARG)
+    Dwarf_Error     * error)
 {
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL inindexptr to"
+            " dwarf_gdbindex_symboltable_array");
+        return DW_DLV_ERROR;
+    }
+    (void)error;
     *list_length = gdbindexptr->gi_symboltablehdr.dg_count;
     return DW_DLV_OK;
 }
@@ -439,20 +626,41 @@ dwarf_gdbindex_symboltable_entry(
     Dwarf_Unsigned * cu_vector_offset,
     Dwarf_Error    * error)
 {
-    Dwarf_Unsigned max =  gdbindexptr->gi_symboltablehdr.dg_count;
+    Dwarf_Unsigned max = 0;
     Dwarf_Small * base = 0;
+    Dwarf_Small * endptr = 0;
     Dwarf_Unsigned symoffset = 0;
     Dwarf_Unsigned cuoffset = 0;
-    unsigned fieldlen = gdbindexptr->gi_symboltablehdr.dg_fieldlen;
+    unsigned fieldlen = 0;
 
-    if (entryindex >= max) {
-        _dwarf_error(gdbindexptr->gi_dbg, error,
-            DW_DLE_GDB_INDEX_INDEX_ERROR);
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL inindexptr to"
+            " dwarf_gdbindex_symboltable_entry");
         return DW_DLV_ERROR;
     }
+    max =  gdbindexptr->gi_symboltablehdr.dg_count;
+    fieldlen = gdbindexptr->gi_symboltablehdr.dg_fieldlen;
+    if (entryindex >= max) {
+        return DW_DLV_NO_ENTRY;
+    }
     base = gdbindexptr->gi_symboltablehdr.dg_base;
     base += entryindex*gdbindexptr->gi_symboltablehdr.dg_entry_length;
+    endptr = gdbindexptr->gi_section_data +
+        gdbindexptr->gi_section_length;
 
+    if (( base + 2*fieldlen) >endptr) {
+        Dwarf_Debug dbg = gdbindexptr->gi_dbg;
+        emit_one_value_msg(dbg, DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " end offset of data for symboltable entry "
+            "%u is past the"
+            " end of the section",
+            entryindex,error);
+        return DW_DLV_ERROR;
+    }
     READ_GDBINDEX(symoffset ,Dwarf_Unsigned,
         base,
         fieldlen);
@@ -465,24 +673,40 @@ dwarf_gdbindex_symboltable_entry(
 }
 
 int
-dwarf_gdbindex_cuvector_length(Dwarf_Gdbindex gdbindex,
+dwarf_gdbindex_cuvector_length(Dwarf_Gdbindex gdbindexptr,
     Dwarf_Unsigned   cuvector_offset,
     Dwarf_Unsigned * innercount,
     Dwarf_Error    * error)
 {
-    Dwarf_Small *base = gdbindex->gi_cuvectorhdr.dg_base;
-    Dwarf_Small *end = gdbindex->gi_section_data +
-        gdbindex->gi_section_length;
+    Dwarf_Small *base = 0;
+    Dwarf_Small *endptr = 0;
+    Dwarf_Unsigned fieldlen = 0;
     Dwarf_Unsigned val = 0;
-    unsigned fieldlen =  gdbindex->gi_cuvectorhdr.dg_entry_length;
 
-    base += cuvector_offset;
-    if ((base + fieldlen) >= end) {
-        _dwarf_error(gdbindex->gi_dbg, error,
-            DW_DLE_GDB_INDEX_INDEX_ERROR);
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL indexptr to"
+            " dwarf_gdbindex_cuvector_length");
         return DW_DLV_ERROR;
     }
 
+    base = gdbindexptr->gi_cuvectorhdr.dg_base;
+    endptr = gdbindexptr->gi_section_data +
+        gdbindexptr->gi_section_length;
+    fieldlen = gdbindexptr->gi_cuvectorhdr.dg_entry_length;
+    base += cuvector_offset;
+    if (( base + fieldlen) >endptr) {
+        Dwarf_Debug dbg = gdbindexptr->gi_dbg;
+        emit_no_value_msg(dbg, DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " end offset of count of gdbindex cuvector "
+            " is past the"
+            " end of the section",
+            error);
+        return DW_DLV_ERROR;
+    }
     READ_GDBINDEX(val,Dwarf_Unsigned,
         base,
         fieldlen);
@@ -499,20 +723,38 @@ dwarf_gdbindex_cuvector_inner_attributes(Dwarf_Gdbindex gdbindexptr,
     Dwarf_Unsigned * attributes,
     Dwarf_Error    * error)
 {
-    Dwarf_Small *base = gdbindexptr->gi_cuvectorhdr.dg_base;
-    Dwarf_Small *end = gdbindexptr->gi_section_data +
-        gdbindexptr->gi_section_length;
+    Dwarf_Small *base = 0;
+    Dwarf_Small *endptr =  0;
+    Dwarf_Unsigned fieldlen = 0;
     Dwarf_Unsigned val = 0;
-    unsigned fieldlen = gdbindexptr->gi_cuvectorhdr.dg_entry_length;
 
-    base += cuvector_offset;
-    if ((base+fieldlen) >= end) {
-        _dwarf_error(gdbindexptr->gi_dbg, error,
-            DW_DLE_GDB_INDEX_INDEX_ERROR);
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        _dwarf_error_string(NULL, error,
+            DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " passed in NULL indexptr to"
+            " dwarf_gdbindex_cuvector_length");
         return DW_DLV_ERROR;
     }
+    base = gdbindexptr->gi_cuvectorhdr.dg_base;
+    base += cuvector_offset;
+    endptr = gdbindexptr->gi_section_data +
+        gdbindexptr->gi_section_length;
+    fieldlen = gdbindexptr->gi_cuvectorhdr.dg_entry_length;
+    /*  The initial 4 bytes is not part of the array,
+        it is some sort of count.  Get past it.*/
     base += fieldlen;
-    base += innerindex*fieldlen;
+    base += fieldlen*innerindex;
+    if ((base+fieldlen) >= endptr) {
+        Dwarf_Debug dbg = gdbindexptr->gi_dbg;
+        emit_one_value_msg(dbg, DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR:"
+            " end offset of data for cuvector_inner_attribute "
+            "%u is past the"
+            " end of the section",
+            innerindex,error);
+        return DW_DLV_ERROR;
+    }
 
     READ_GDBINDEX(val ,Dwarf_Unsigned,
         base,
@@ -541,6 +783,7 @@ dwarf_gdbindex_cuvector_instance_expand_value(
     *symbol_kind = (value >> 28) & 0x7;
     *is_static =   (value >> 31) & 1;
     return DW_DLV_OK;
+
 }
 
 /*  The strings in the pool follow (in memory) the cu index
@@ -555,58 +798,40 @@ dwarf_gdbindex_string_by_offset(Dwarf_Gdbindex gdbindexptr,
     Dwarf_Small *section_end = 0;
     Dwarf_Small *stringitself = 0;
     Dwarf_Debug dbg = 0;
+    Dwarf_Unsigned fulloffset = 0;
     int res = 0;
 
-    if (!gdbindexptr) {
-        dwarfstring m;
-
-        dwarfstring_constructor(&m);
-        dwarfstring_append(&m,"DW_DLE_GDB_INDEX_INDEX_ERROR: "
+    if (!gdbindexptr || !gdbindexptr->gi_dbg) {
+        emit_no_value_msg(NULL,DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR: "
             "The gdbindex pointer to "
             "dwarf_gdbindex_string_by_offset()"
-            " is NULL");
-        _dwarf_error_string(NULL, error,
-            DW_DLE_GDB_INDEX_INDEX_ERROR,
-            dwarfstring_string(&m));
-        dwarfstring_destructor(&m);
+            " is NULL",error);
         return DW_DLV_ERROR;
     }
     dbg = gdbindexptr->gi_dbg;
-    if (!dbg) {
-        dwarfstring m;
-
-        dwarfstring_constructor(&m);
-        dwarfstring_append(&m,"DW_DLE_GDB_INDEX_INDEX_ERROR: "
+    if (IS_INVALID_DBG(dbg)) {
+        emit_no_value_msg(NULL,DW_DLE_GDB_INDEX_INDEX_ERROR,
+            "DW_DLE_GDB_INDEX_INDEX_ERROR: "
             "The gdbindex Dwarf_Debug in"
             "dwarf_gdbindex_string_by_offset()"
-            " is NULL");
-        _dwarf_error_string(dbg, error,
-            DW_DLE_GDB_INDEX_INDEX_ERROR,
-            dwarfstring_string(&m));
-        dwarfstring_destructor(&m);
+            " is NULL",error);
         return DW_DLV_ERROR;
     }
-    pooldata = gdbindexptr->gi_section_data +
-        gdbindexptr->gi_constant_pool_offset;
     section_end = gdbindexptr->gi_section_data +
         gdbindexptr->gi_section_length;
-    stringitself = pooldata + stringoffsetinpool;
+    fulloffset = gdbindexptr->gi_constant_pool_offset
+        + stringoffsetinpool;
+    stringitself = gdbindexptr->gi_section_data + fulloffset;
     if (stringitself > section_end) {
-        dwarfstring m;
-
-        dwarfstring_constructor(&m);
-        dwarfstring_append_printf_u(&m,
+        emit_one_value_msg(dbg,DW_DLE_GDBINDEX_STRING_ERROR,
             "DW_DLE_GDBINDEX_STRING_ERROR: "
             "The dwarf_gdbindex_string_by_offset() "
             "string starts past the end of the "
             "section at section_offset 0x%"
             DW_PR_XZEROS DW_PR_DUx  ".",
-            (Dwarf_Unsigned)(uintptr_t)
-            (stringitself -gdbindexptr->gi_section_data));
-        _dwarf_error_string(dbg, error,
-            DW_DLE_GDBINDEX_STRING_ERROR,
-            dwarfstring_string(&m));
-        dwarfstring_destructor(&m);
+            fulloffset,
+            error);
         return DW_DLV_ERROR;
     }
     res = _dwarf_check_string_valid(dbg,pooldata,
@@ -621,7 +846,7 @@ dwarf_gdbindex_string_by_offset(Dwarf_Gdbindex gdbindexptr,
 }
 
 void
-dwarf_gdbindex_free(Dwarf_Gdbindex indexptr)
+dwarf_dealloc_gdbindex(Dwarf_Gdbindex indexptr)
 {
     if (indexptr) {
         Dwarf_Debug dbg = indexptr->gi_dbg;
