@@ -1,4 +1,4 @@
-/* Copyright (c) 2013-2021, David Anderson
+/* Copyright (c) 2013-2022, David Anderson
 All rights reserved.
 
 Redistribution and use in source and binary forms, with
@@ -54,6 +54,9 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     The 'preorder' etc labels mean nothing in a hash, so everything
     is called a leaf.
 
+    Since libdwarf never calls dwarf_tdump we
+    never define BUILD_TDUMP
+
 */
 
 #include <config.h>
@@ -67,6 +70,7 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endif /* HAVE_STDINT_H */
 
 #include "libdwarf.h"
+#include "dwarf_local_malloc.h"
 #include "libdwarf_private.h"
 #include "dwarf_tsearch.h"
 
@@ -78,8 +82,9 @@ static unsigned long primes[] =
 {
 #if 0 /* for testing only */
 5,11, 17,23, 31, 47, 53,
-#endif /*0*/
 79,
+#endif /*0*/
+521,
 1009,
 5591,
 10007,
@@ -137,14 +142,13 @@ enum search_intent_t
 };
 
 static struct ts_entry *
-tsearch_inner( const void *key, struct hs_base* head,
+_tsearch_inner( const void *key, struct hs_base* head,
     int (*compar)(const void *, const void *),
     const enum search_intent_t intent, int*inserted,
     struct ts_entry **parent_ptr);
 static void
-dwarf_tdestroy_inner(struct hs_base*h,
-    void (*free_node)(void *nodep),
-    int depth);
+_dwarf_tdestroy_inner(struct hs_base*h,
+    void (*free_node)(void *nodep));
 
 /*  A trivial integer-based percentage calculation.
     Percents >100 are reasonable for a hash-with-chains
@@ -187,7 +191,7 @@ dwarf_initialize_search_hash( void **treeptr,
         /* initialized already. */
         return base ;
     }
-    base = calloc(sizeof(struct hs_base),1);
+    base = calloc(1, sizeof(struct hs_base));
     if (!base) {
         /* Out of memory. */
         return NULL ;
@@ -203,6 +207,10 @@ dwarf_initialize_search_hash( void **treeptr,
         }
         entry_index = k;
     }
+#ifdef TESTINGHASHTAB
+printf("debugging: initial alloc size estimate %lu\n",size_estimate);
+printf("debugging: initial alloc prime to use %lu\n",prime_to_use);
+#endif
     base->tablesize_ = prime_to_use;
     base->allowed_fill_ = calculate_allowed_fill(allowed_fill_percent,
         prime_to_use);
@@ -216,7 +224,8 @@ dwarf_initialize_search_hash( void **treeptr,
     /*  hashtab_ is an array of hs_entry,
         indexes 0 through tablesize_ -1. */
     base->hashfunc_ = hashfunc;
-    base->hashtab_ = calloc(sizeof(struct ts_entry),base->tablesize_);
+    base->hashtab_ = calloc(base->tablesize_,
+        sizeof(struct ts_entry));
     if (!base->hashtab_) {
         free(base);
         return NULL;
@@ -225,6 +234,7 @@ dwarf_initialize_search_hash( void **treeptr,
     return base;
 }
 
+#ifdef BUILD_TDUMP
 /*  We don't really care whether hashpos or chainpos
     are 32 or 64 bits. 32 suffices. */
 static void print_entry(struct ts_entry *t,const char *descr,
@@ -315,6 +325,7 @@ dwarf_tdump(const void*headp_in,
     }
     dumptree_inner(head,keyprint,msg,1);
 }
+#endif /* BUILD_TDUMP */
 
 static struct ts_entry *
 allocate_ts_entry(const void *key)
@@ -359,8 +370,8 @@ resize_table(struct hs_base *head,
         return;
     }
     newhead.tablesize_entry_index_ = new_entry_index;
-    newhead.hashtab_ = calloc(sizeof(struct ts_entry),
-        newhead.tablesize_);
+    newhead.hashtab_ = calloc(newhead.tablesize_,
+        sizeof(struct ts_entry));
     if (!newhead.hashtab_) {
         /*  Oops, too large. Leave table size as is, though
             things will get slow as it overfills. */
@@ -374,6 +385,9 @@ resize_table(struct hs_base *head,
         unsigned long ix = 0;
         unsigned long tsize = head->tablesize_;
         struct ts_entry *p = &head->hashtab_[0];
+#ifdef TESTINGHASHTAB
+printf("debugging: Resize %lu to %lu\n",tsize,prime_to_use);
+#endif
         for ( ; ix < tsize; ix++,p++) {
             int inserted = 0;
             struct ts_entry*n = 0;
@@ -381,7 +395,7 @@ resize_table(struct hs_base *head,
                 break;
             }
             if (p->keyptr) {
-                tsearch_inner(p->keyptr,
+                _tsearch_inner(p->keyptr,
                     &newhead,compar,
                     want_insert,
                     &inserted,
@@ -393,7 +407,7 @@ resize_table(struct hs_base *head,
             }
             for (n = p->next; n ; n = n->next) {
                 inserted = 0;
-                tsearch_inner(n->keyptr,
+                _tsearch_inner(n->keyptr,
                     &newhead,compar,
                     want_insert,
                     &inserted,
@@ -410,7 +424,7 @@ resize_table(struct hs_base *head,
         }
     }
     /* Now get rid of the chain entries of the old table. */
-    dwarf_tdestroy_inner(head,0,0);
+    _dwarf_tdestroy_inner(head,0);
     /* Now get rid of the old table itself. */
     free(head->hashtab_);
     head->hashtab_ = 0;
@@ -420,7 +434,7 @@ resize_table(struct hs_base *head,
 
 /*  Inner search of the hash and synonym chains.  */
 static struct ts_entry *
-tsearch_inner( const void *key, struct hs_base* head,
+_tsearch_inner( const void *key, struct hs_base* head,
     int (*compar)(const void *, const void *),
     const enum search_intent_t intent, int*inserted,
     /* owner_ptr used for delete.  Only set
@@ -466,7 +480,7 @@ tsearch_inner( const void *key, struct hs_base* head,
     kc = compar(key,s->keyptr);
     if (kc == 0 ) {
         /* found! */
-        if (want_delete) {
+        if (intent == want_delete) {
             *owner_ptr = 0;
         }
         return (void *)&(s->keyptr);
@@ -476,7 +490,7 @@ tsearch_inner( const void *key, struct hs_base* head,
         kc = compar(key,c->keyptr);
         if (kc == 0 ) {
             /* found! */
-            if (want_delete) {
+            if (intent == want_delete) {
                 *owner_ptr = chain_parent;
             }
             return (void *)&(c->keyptr);
@@ -513,7 +527,7 @@ dwarf_tsearch(const void *key, void **headin,
         /* something is wrong here, not initialized. */
         return NULL;
     }
-    r = tsearch_inner(key,head,compar,want_insert,&inserted,&nullme);
+    r = _tsearch_inner(key,head,compar,want_insert,&inserted,&nullme);
     if (!r) {
         return NULL;
     }
@@ -540,7 +554,7 @@ dwarf_tfind(const void *key, void *const *rootp,
         return NULL;
     }
 
-    r = tsearch_inner(key,head,compar,only_find,&inserted,&nullme);
+    r = _tsearch_inner(key,head,compar,only_find,&inserted,&nullme);
     if (!r) {
         return NULL;
     }
@@ -565,7 +579,7 @@ dwarf_tdelete(const void *key, void **rootp,
         return NULL;
     }
 
-    found = tsearch_inner(key,head,compar,want_delete,&inserted,
+    found = _tsearch_inner(key,head,compar,want_delete,&inserted,
         &parentp);
     if (found) {
         if (parentp) {
@@ -605,21 +619,22 @@ dwarf_tdelete(const void *key, void **rootp,
 }
 
 static void
-dwarf_twalk_inner(const struct hs_base *h,
+_dwarf_twalk_inner(const struct hs_base *h,
     struct ts_entry *p,
     void (*action)(const void *nodep, const DW_VISIT which,
-        const int depth UNUSEDARG),
-    unsigned level UNUSEDARG)
+        const int depth )
+    )
 {
     unsigned long ix = 0;
+    int depth = 0;
     unsigned long tsize = h->tablesize_;
     for ( ; ix < tsize; ix++,p++) {
         struct ts_entry*n = 0;
         if (p->keyptr) {
-            action((void *)(&(p->keyptr)),dwarf_leaf,level);
+            action((void *)(&(p->keyptr)),dwarf_leaf,depth);
         }
         for (n = p->next; n ; n = n->next) {
-            action((void *)(&(n->keyptr)),dwarf_leaf,level);
+            action((void *)(&(n->keyptr)),dwarf_leaf,depth);
         }
     }
 }
@@ -627,7 +642,7 @@ dwarf_twalk_inner(const struct hs_base *h,
 void
 dwarf_twalk(const void *rootp,
     void (*action)(const void *nodep, const DW_VISIT which,
-        const int depth UNUSEDARG))
+        const int depth))
 {
     const struct hs_base *head = (const struct hs_base *)rootp;
     struct ts_entry *root = 0;
@@ -636,17 +651,21 @@ dwarf_twalk(const void *rootp,
     }
     root = head->hashtab_;
     /* Get to actual tree. */
-    dwarf_twalk_inner(head,root,action,0);
+    _dwarf_twalk_inner(head,root,action);
 }
 
 static void
-dwarf_tdestroy_inner(struct hs_base*h,
-    void (*free_node)(void *nodep),
-    int depth UNUSEDARG)
+_dwarf_tdestroy_inner(struct hs_base*h,
+    void (*free_node)(void *nodep))
 {
     unsigned long ix = 0;
     unsigned long tsize = h->tablesize_;
     struct ts_entry *p = &h->hashtab_[0];
+#ifdef TESTINGHASHTAB
+    printf("debugging: destroyhashtable blocks      %lu\n",tsize);
+    printf("debugging: destroyhashtable recordcount %lu\n",
+        h->record_count_);
+#endif
     for ( ; ix < tsize; ix++,p++) {
         struct ts_entry*n = 0;
         struct ts_entry*prev = 0;
@@ -685,7 +704,7 @@ dwarf_tdestroy(void *rootp, void (*free_node)(void *nodep))
         return;
     }
     root = head->hashtab_;
-    dwarf_tdestroy_inner(head,free_node,0);
+    _dwarf_tdestroy_inner(head,free_node);
     free(root);
     free(head);
 }
