@@ -67,6 +67,7 @@
 #include <config.h>
 
 #include <string.h>  /* memcmp() memcpy() strcmp() */
+#include <stdio.h>  /*  for printf (debugging) */
 
 #if defined(_WIN32) && defined(HAVE_STDAFX_H)
 #include "stdafx.h"
@@ -74,6 +75,7 @@
 
 #include "dwarf.h"
 #include "libdwarf.h"
+#include "dwarf_local_malloc.h"
 #include "libdwarf_private.h"
 #include "dwarf_base_types.h"
 #include "dwarf_opaque.h"
@@ -88,14 +90,14 @@
 /* zerohashkey used as all-zero-bits for comparison. */
 static const Dwarf_Sig8 zerohashkey;
 
-#if 0
+#if 0 /* dump_bytes */
 static void
 dump_bytes(char * msg,Dwarf_Small * start, long len)
 {
     Dwarf_Small *end = start + len;
     Dwarf_Small *cur = start;
 
-    printf("%s ",msg);
+    printf("%s %p: ",msg, (void *)start);
     for (; cur < end; cur++) {
         printf("%02x ", *cur);
     }
@@ -110,7 +112,7 @@ fill_in_offsets_headerline(Dwarf_Debug dbg,
     Dwarf_Xu_Index_Header xuhdr,
     Dwarf_Unsigned headerline_offset,
     Dwarf_Unsigned num_sects,
-    Dwarf_Error *err)
+    Dwarf_Error *error)
 {
     Dwarf_Small *section_start = xuhdr->gx_section_data;
     Dwarf_Small *section_end = xuhdr->gx_section_data+
@@ -124,7 +126,7 @@ fill_in_offsets_headerline(Dwarf_Debug dbg,
 
         READ_UNALIGNED_CK(dbg,v, Dwarf_Unsigned,
             data,SIZEOFT32,
-            err,section_end);
+            error,section_end);
         data += SIZEOFT32;
         if (v > DW_SECT_RNGLISTS) {
             dwarfstring s;
@@ -136,12 +138,12 @@ fill_in_offsets_headerline(Dwarf_Debug dbg,
             dwarfstring_append(&s," is too high. "
                 "Sections 1-8 are listed in "
                 "DWARF5 Table 7.1.");
-            _dwarf_error_string(dbg, err, DW_DLE_XU_NAME_COL_ERROR,
+            _dwarf_error_string(dbg, error, DW_DLE_XU_NAME_COL_ERROR,
                 dwarfstring_string(&s));
             dwarfstring_destructor(&s);
             return DW_DLV_ERROR;
         }
-        xuhdr->gx_section_id[i] = v;
+        xuhdr->gx_section_id[i] = (unsigned long)v;
     }
     return DW_DLV_OK;
 }
@@ -153,7 +155,7 @@ fill_in_offsets_headerline(Dwarf_Debug dbg,
     dwarf_init*() calls
     dwarf_get_xu_index_header() when
     the object file is opened and
-    dwarf_xu_header_free() is called
+    dwarf_dealloc_xu_header() is called
     by dwarf_finish(), there is
     no need for users to do this.
 
@@ -161,7 +163,7 @@ fill_in_offsets_headerline(Dwarf_Debug dbg,
     tu/cu functions oneself (possibly to print the
     .debug_cu_index or .debug_tu_index sections).
     then you will need to call dwarf_get_xu_index_header()
-    and eventually dwarf_xu_header_free().
+    and eventually dwarf_dealloc_xu_header().
 
     The libdwarf-internal data is kept in Dwarf_Debug
     fields de_cu_hashindex_data/de_tu_hashindex_data.
@@ -169,15 +171,15 @@ fill_in_offsets_headerline(Dwarf_Debug dbg,
 int
 dwarf_get_xu_index_header(Dwarf_Debug dbg,
     /* Pass in section_type "tu" or "cu" */
-    const char *     section_type,
-    Dwarf_Xu_Index_Header * xuptr,
-    Dwarf_Unsigned * version,
-    Dwarf_Unsigned * number_of_columns, /* L section count.*/
-    Dwarf_Unsigned * number_of_CUs,     /* U unit count    */
-    Dwarf_Unsigned * number_of_slots,   /* S slot count    */
+    const char     *section_type,
+    Dwarf_Xu_Index_Header *xuptr,
+    Dwarf_Unsigned *version,
+    Dwarf_Unsigned *number_of_columns, /* L section count.*/
+    Dwarf_Unsigned *number_of_CUs,     /* U unit count    */
+    Dwarf_Unsigned *number_of_slots,   /* S slot count    */
     /*  Standard says S > U DWARF5 sec 7.3.5.3 */
-    const char    ** section_name,
-    Dwarf_Error    * error)
+    const char    **section_name,
+    Dwarf_Error    *error)
 {
     Dwarf_Xu_Index_Header indexptr = 0;
     int res = DW_DLV_ERROR;
@@ -186,7 +188,7 @@ dwarf_get_xu_index_header(Dwarf_Debug dbg,
     Dwarf_Unsigned num_secs  = 0;
     Dwarf_Unsigned num_CUs  = 0;
     Dwarf_Unsigned num_slots  = 0;
-    Dwarf_Small *data = 0;
+    Dwarf_Small   *data = 0;
     Dwarf_Unsigned tables_end_offset = 0;
     Dwarf_Unsigned hash_tab_offset = 0;
     Dwarf_Unsigned indexes_tab_offset = 0;
@@ -196,12 +198,22 @@ dwarf_get_xu_index_header(Dwarf_Debug dbg,
     unsigned datalen32 = SIZEOFT32;
     Dwarf_Small *section_end = 0;
 
+    CHECK_DBG(dbg,error,"dwarf_get_xu_index_header()");
+    if (!section_type || !xuptr) {
+        _dwarf_error_string(0,error,DW_DLE_XU_TYPE_ARG_ERROR,
+            "DW_DLE_XU_TYPE_ARG_ERROR: section type or header "
+            "return pointer is not valid");
+        return DW_DLV_ERROR;
+    }
+
     if (!strcmp(section_type,"cu") ) {
         sect = &dbg->de_debug_cu_index;
     } else if (!strcmp(section_type,"tu") ) {
         sect = &dbg->de_debug_tu_index;
     } else {
-        _dwarf_error(dbg, error, DW_DLE_XU_TYPE_ARG_ERROR);
+        _dwarf_error_string(dbg, error, DW_DLE_XU_TYPE_ARG_ERROR,
+            "DW_DLE_XU_TYPE_ARG_ERROR, Passed in section type "
+            "is neither \"tu\" nor \"cu\"");
         return DW_DLV_ERROR;
     }
     if (!sect->dss_size) {
@@ -239,8 +251,23 @@ dwarf_get_xu_index_header(Dwarf_Debug dbg,
         data,datalen32,
         error,section_end);
     data += datalen32;
+    if (local_version < DW_CU_VERSION2 ||
+        local_version > DW_CU_VERSION5) {
+        /*  A GNU extension allowed debug_fission to
+            apply to DWARF4, and even down to...DWARF2! */
+        dwarfstring m;
 
-    /* reading N */
+        dwarfstring_constructor(&m);
+
+        dwarfstring_append_printf_u(&m,"DW_DLE_XU_NAME_COL_ERROR: "
+            "The CU/TU Index Section version is %u ",local_version);
+        _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+            dwarfstring_string(&m));
+        dwarfstring_destructor(&m);
+        return DW_DLV_ERROR;
+    }
+
+    /* reading N, section_count */
     READ_UNALIGNED_CK(dbg,num_secs, Dwarf_Unsigned,
         data,datalen32,
         error,section_end);
@@ -263,12 +290,21 @@ dwarf_get_xu_index_header(Dwarf_Debug dbg,
         return DW_DLV_ERROR;
     }
     data += datalen32;
-    /* reading U */
+    /*  Reading U, unit_count, compilation-units or type-units
+        depending on cu or tu */
     READ_UNALIGNED_CK(dbg,num_CUs, Dwarf_Unsigned,
         data,datalen32,
         error,section_end);
     data += datalen32;
-    /* reading S */
+    if (num_CUs >= sect->dss_size ||
+        (num_CUs/datalen32) >= sect->dss_size) {
+        _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+            "dbg,error,DW_DLE_XU_NAME_COL_ERROR: "
+            "The CU/TU Index Section header unit count "
+            "is too large to be real. Corrupt DWARF\n");
+        return DW_DLV_ERROR;
+    }
+    /* reading S, slot_count, slots in the hash table */
     READ_UNALIGNED_CK(dbg,num_slots, Dwarf_Unsigned,
         data,datalen32,
         error,section_end);
@@ -422,13 +458,14 @@ int dwarf_get_xu_index_section_type(Dwarf_Xu_Index_Header xuhdr,
     /*  the function returns a pointer to
         the immutable string "tu" or "cu" via
         this arg. Do not free.  */
-    const char ** typename,
+    const char **typename,
     /*  the function returns a pointer to
         the immutable section name. Do not free.
         .debug_cu_index or .debug_tu_index */
-    const char ** sectionname,
-    UNUSEDARG Dwarf_Error * err)
+    const char **sectionname,
+    Dwarf_Error *error)
 {
+    (void)error;
     *typename    = &xuhdr->gx_type[0];
     *sectionname = xuhdr->gx_section_name;
     return DW_DLV_OK;
@@ -436,13 +473,13 @@ int dwarf_get_xu_index_section_type(Dwarf_Xu_Index_Header xuhdr,
 
 /*  Index values 0 to S-1 are valid. */
 int dwarf_get_xu_hash_entry(Dwarf_Xu_Index_Header xuhdr,
-    Dwarf_Unsigned    index,
+    Dwarf_Unsigned  index,
     /* returns the hash value. 64 bits. */
-    Dwarf_Sig8     *  hash_value,
+    Dwarf_Sig8     *hash_value,
 
     /* returns the index into rows of offset/size tables. */
-    Dwarf_Unsigned *  index_to_sections,
-    Dwarf_Error *     err)
+    Dwarf_Unsigned *index_to_sections,
+    Dwarf_Error    *error)
 {
     Dwarf_Debug dbg = xuhdr->gx_dbg;
     Dwarf_Small *hashtab = xuhdr->gx_section_data +
@@ -456,33 +493,36 @@ int dwarf_get_xu_hash_entry(Dwarf_Xu_Index_Header xuhdr,
         xuhdr->gx_section_length;
 
     if (!xuhdr->gx_slots_in_hash) {
-        _dwarf_error_string(dbg, err,  DW_DLE_XU_HASH_ROW_ERROR,
+        _dwarf_error_string(dbg, error,  DW_DLE_XU_HASH_ROW_ERROR,
             "DW_DLE_XU_HASH_ROW_ERROR the number of slots is zero"
             " which seems wrong.");
         return DW_DLV_ERROR;
     }
     if (index >= xuhdr->gx_slots_in_hash) {
+        return DW_DLV_NO_ENTRY;
+    }
+    hashentry = hashtab + (index * HASHSIGNATURELEN);
+    memcpy(hash_value,hashentry,HASHSIGNATURELEN);
+    indexentry = indextab + (index * SIZEOFT32);
+    if (indexentry >= section_end ||
+        (indexentry+SIZEOFT32) > section_end) {
         dwarfstring m;
 
         dwarfstring_constructor(&m);
         dwarfstring_append_printf_u(&m,
             "DW_DLE_XU_HASH_ROW_ERROR the index passed in, "
-            " %u, is greater than the number of slots "
-            " in the hash table.",index);
-        _dwarf_error_string(dbg, err,  DW_DLE_XU_HASH_ROW_ERROR,
+            " %u, results in entry not fitting"
+            " in the hash table (past end of section)",index);
+        _dwarf_error_string(dbg, error,  DW_DLE_XU_HASH_ROW_ERROR,
             dwarfstring_string(&m));
         dwarfstring_destructor(&m);
         return DW_DLV_ERROR;
     }
-    hashentry = hashtab + (index * HASHSIGNATURELEN);
-    memcpy(hash_value,hashentry,HASHSIGNATURELEN);
-    indexentry = indextab + (index * SIZEOFT32);
     READ_UNALIGNED_CK(dbg,indexval,Dwarf_Unsigned, indexentry,
         SIZEOFT32,
-        err,section_end);
-    indexentry += SIZEOFT32;
+        error,section_end);
     if (indexval > xuhdr->gx_units_in_index) {
-        _dwarf_error(dbg, err,  DW_DLE_XU_HASH_INDEX_ERROR);
+        _dwarf_error(dbg, error,  DW_DLE_XU_HASH_INDEX_ERROR);
         return DW_DLV_ERROR;
     }
     *index_to_sections = indexval;
@@ -495,25 +535,22 @@ static const char * dwp_secnames[] = {
 "DW_SECT_TYPES"       /* 2 */ /*".debug_types.dwo"*/,
 "DW_SECT_ABBREV"      /* 3 */ /*".debug_abbrev.dwo"*/,
 "DW_SECT_LINE"        /* 4 */ /*".debug_line.dwo"*/,
-"DW_SECT_LOC"         /* 5 */ /*".debug_loc.dwo"*/,
+"DW_SECT_LOC"         /* 5 */ /*".debug_loclists.dwo"*/,
 "DW_SECT_STR_OFFSETS" /* 6 */ /*".debug_str_offsets.dwo"*/,
 "DW_SECT_MACRO"       /* 7 */ /*".debug_macro.dwo"*/,
 "DW_SECT_RNGLISTS"       /* 8 */ /*".debug_rnglists.dwo"*/,
 "No name > 8",
 };
 
-/*  Row 0 of the Table of Section Offsets,
-    columns 0 to N-1,  are the section id's,
-    and names, such as DW_SECT_INFO (ie, 1)  */
-int
-dwarf_get_xu_section_names(Dwarf_Xu_Index_Header xuhdr,
+static int
+_dwarf_find_xu_target_sec_size(Dwarf_Debug dbg,
+    Dwarf_Xu_Index_Header xuhdr,
     Dwarf_Unsigned  column_index,
-    Dwarf_Unsigned* number,
-    const char **   name,
-    Dwarf_Error *   err)
+    Dwarf_Unsigned  *target_sec_size,
+    Dwarf_Error *error)
 {
     Dwarf_Unsigned sec_num = 0;
-    Dwarf_Debug dbg = xuhdr->gx_dbg;
+    struct  Dwarf_Section_s *secptr = 0;
 
     if ( column_index >= xuhdr->gx_column_count_sections) {
         dwarfstring s;
@@ -525,7 +562,117 @@ dwarf_get_xu_section_names(Dwarf_Xu_Index_Header xuhdr,
         dwarfstring_append_printf_u(&s," is too high. "
             "There are %u sections.",
             xuhdr->gx_column_count_sections);
-        _dwarf_error_string(dbg, err, DW_DLE_XU_NAME_COL_ERROR,
+        _dwarf_error_string(dbg, error, DW_DLE_XU_NAME_COL_ERROR,
+            dwarfstring_string(&s));
+        dwarfstring_destructor(&s);
+        return DW_DLV_ERROR;
+    }
+    sec_num = xuhdr->gx_section_id[column_index];
+    switch(sec_num) {
+    case DW_SECT_INFO:
+        secptr = &dbg->de_debug_info;
+        break;
+    case DW_SECT_TYPES:
+        secptr = &dbg->de_debug_types;
+        break;
+    case DW_SECT_ABBREV:
+        secptr = &dbg->de_debug_abbrev;
+        break;
+    case DW_SECT_LINE:
+        secptr = &dbg->de_debug_line;
+        break;
+    case DW_SECT_LOCLISTS:
+        /* May not be loaded */
+        if (xuhdr->gx_version < DW_CU_VERSION5) {
+            secptr = &dbg->de_debug_loc;
+        } else {
+            secptr = &dbg->de_debug_loclists;
+        }
+        break;
+    case DW_SECT_STR_OFFSETS:
+        /* May not be loaded */
+        secptr = &dbg->de_debug_str_offsets;
+        break;
+    case DW_SECT_MACRO:
+        /* May not be loaded */
+        secptr = &dbg->de_debug_str_offsets;
+        break;
+    case DW_SECT_RNGLISTS:
+        /* May not be loaded */
+        if (xuhdr->gx_version < DW_CU_VERSION5) {
+            secptr = &dbg->de_debug_ranges;
+        } else {
+            secptr = &dbg->de_debug_rnglists;
+        }
+        break;
+    default:
+        {
+            dwarfstring s;
+
+            dwarfstring_constructor(&s);
+            dwarfstring_append_printf_u(&s,
+                "DW_DLE_XU_NAME_COL_ERROR the .debug_cu/tu_index "
+                "section has an unexpected DW_SECT  number "
+                "of %u.",sec_num);
+            _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+                dwarfstring_string(&s));
+            dwarfstring_destructor(&s);
+            return DW_DLV_ERROR;
+        }
+    }
+    if (!secptr->dss_size) {
+        int res = 0;
+
+        /*  Perhaps the section not yet loaded. */
+        res = _dwarf_load_section(dbg,secptr,error);
+        if (res == DW_DLV_ERROR) {
+            dwarf_dealloc_error(dbg,*error);
+            *error = 0;
+            return DW_DLV_NO_ENTRY;
+        } else {
+            if (res == DW_DLV_NO_ENTRY) {
+                /*  Sometimes GNU extension ( DWARF2-4)
+                    is simply missing a section named in fission */
+                return res;
+            }
+        }
+    }
+    *target_sec_size =secptr->dss_size;
+    return DW_DLV_OK;
+}
+
+/*  Row 0 of the Table of Section Offsets,
+    columns 0 to N-1,  are the section id's,
+    and names, such as DW_SECT_INFO (ie, 1)  */
+int
+dwarf_get_xu_section_names(Dwarf_Xu_Index_Header xuhdr,
+    Dwarf_Unsigned  column_index,
+    Dwarf_Unsigned *number,
+    const char    **name,
+    Dwarf_Error    *error)
+{
+    Dwarf_Unsigned sec_num = 0;
+    Dwarf_Debug dbg = 0;
+    if (!xuhdr) {
+        _dwarf_error_string(0,error,DW_DLE_XU_TYPE_ARG_ERROR,
+            "DW_DLE_XU_TYPE_ARG_ERROR: "
+            "Dwarf_Xu_Index_Header is NULL");
+        return DW_DLV_ERROR;
+    }
+    dbg = xuhdr->gx_dbg;
+/* FIXME */
+    CHECK_DBG(dbg,error,"dwarf_get_xu_section_names()");
+    if ( column_index >= xuhdr->gx_column_count_sections) {
+        dwarfstring s;
+
+        dwarfstring_constructor(&s);
+        dwarfstring_append_printf_u(&s,
+            "ERROR: DW_DLE_XU_NAME_COL_ERROR as the "
+            "column index of %u ",column_index);
+        dwarfstring_append_printf_u(&s," is too high. "
+            "There are %u sections.",
+            xuhdr->gx_column_count_sections);
+        _dwarf_error_string(dbg, error, DW_DLE_XU_NAME_COL_ERROR,
             dwarfstring_string(&s));
         dwarfstring_destructor(&s);
         return DW_DLV_ERROR;
@@ -539,38 +686,80 @@ dwarf_get_xu_section_names(Dwarf_Xu_Index_Header xuhdr,
     return DW_DLV_OK;
 }
 
+static Dwarf_Bool
+_dwarf_valid_col_index(Dwarf_Unsigned i,Dwarf_Unsigned count)
+{
+    if (i > count) {
+        return FALSE;
+    }
+    return TRUE;
+}
 /*  Rows 0 to U-1
     col 0 to L-1
     are section offset and length values from
     the Table of Section Offsets and Table of Section Sizes.
     The formally the table of section offsets is a header
     line of the section offsets we subtract 1 from
-    the incoming irow_index as our tables are
-    now zero origin. */
+    the incoming irow_index as our tables (irow_index and
+    column_index are now zero origin. */
 int
 dwarf_get_xu_section_offset(Dwarf_Xu_Index_Header xuhdr,
-    Dwarf_Unsigned  irow_index,
-    Dwarf_Unsigned  column_index,
-    Dwarf_Unsigned* sec_offset,
-    Dwarf_Unsigned* sec_size,
-    Dwarf_Error *   err)
+    Dwarf_Unsigned  irow_index  /* one-origin */,
+    Dwarf_Unsigned  column_index/* zero-origin */,
+    Dwarf_Unsigned *sec_offset,
+    Dwarf_Unsigned *sec_size,
+    Dwarf_Error    *error)
 {
     /* We use zero origin in the arrays, Users see
         one origin from the hash table. */
-    Dwarf_Debug dbg = xuhdr->gx_dbg;
+    Dwarf_Debug    dbg = 0;
     /* get to base of tables first. */
-    Dwarf_Small *offsetrow =  xuhdr->gx_section_offsets_offset +
-        xuhdr->gx_section_data;
-    Dwarf_Small *sizerow =  xuhdr->gx_section_sizes_offset +
-        xuhdr->gx_section_data;
-    Dwarf_Small *offsetentry = 0;
-    Dwarf_Small *sizeentry =  0;
+    Dwarf_Small   *offsetrow =  0;
+    Dwarf_Small   *sizerow =  0;
+    Dwarf_Small   *offsetentry = 0;
+    Dwarf_Small   *sizeentry =  0;
     Dwarf_Unsigned offset = 0;
     Dwarf_Unsigned size = 0;
-    Dwarf_Unsigned column_count = xuhdr->gx_column_count_sections;
-    Dwarf_Small *section_end = xuhdr->gx_section_data +
-        xuhdr->gx_section_length;
+    Dwarf_Unsigned column_count = 0;
+    Dwarf_Small   *section_end = 0;
     Dwarf_Unsigned row_index = irow_index-1;
+    Dwarf_Unsigned secsize = 0;
+    /*  These 4 are to help in sanity checks.*/
+    Dwarf_Unsigned ofrow = 0;
+    Dwarf_Unsigned ofentry = 0;
+    Dwarf_Unsigned sirow = 0;
+    Dwarf_Unsigned sientry = 0;
+    int            res     = 0;
+    Dwarf_Unsigned target_sec_size;
+
+    if (!xuhdr) {
+        _dwarf_error_string(0,error,DW_DLE_XU_TYPE_ARG_ERROR,
+            "DW_DLE_XU_TYPE_ARG_ERROR: "
+            "Dwarf_Xu_Index_Header pointer is null");
+        return DW_DLV_ERROR;
+    }
+    dbg = xuhdr->gx_dbg;
+    CHECK_DBG(dbg,error,"dwarf_get_xu_section_offset()");
+    sizerow =  xuhdr->gx_section_sizes_offset +
+        xuhdr->gx_section_data;
+    offsetrow =  xuhdr->gx_section_offsets_offset +
+        xuhdr->gx_section_data;
+    column_count = xuhdr->gx_column_count_sections;
+    secsize =  xuhdr->gx_section_length;
+    section_end = xuhdr->gx_section_data +
+        xuhdr->gx_section_length;
+    if (!_dwarf_valid_col_index(column_index,column_count)) {
+        dwarfstring s;
+
+        dwarfstring_constructor(&s);
+        dwarfstring_append_printf_u(&s,"DW_DLE_XU_TYPE_ARG_ERROR: "
+            " dwarf_get_xu_section_offset was passed an invalid"
+            " cu/tu column index of %u",column_index);
+        _dwarf_error_string(dbg,error,DW_DLE_XU_TYPE_ARG_ERROR,
+            dwarfstring_string(&s));
+        dwarfstring_destructor(&s);
+        return DW_DLV_ERROR;
+    }
 
     if (!irow_index) {
         dwarfstring s;
@@ -582,11 +771,12 @@ dwarf_get_xu_section_offset(Dwarf_Xu_Index_Header xuhdr,
             "is zero, which is not a valid row in "
             " the offset-table or the size table as we think"
             " of them as 1-origin.");
-        _dwarf_error_string(dbg, err, DW_DLE_XU_NAME_COL_ERROR,
+        _dwarf_error_string(dbg, error, DW_DLE_XU_NAME_COL_ERROR,
             dwarfstring_string(&s));
         dwarfstring_destructor(&s);
         return DW_DLV_ERROR;
     }
+    /*  Having subtracted 1, check the zero-origin value */
     if (row_index >= xuhdr->gx_units_in_index) {
         dwarfstring s;
 
@@ -596,13 +786,13 @@ dwarf_get_xu_section_offset(Dwarf_Xu_Index_Header xuhdr,
             "row index of %u ",row_index);
         dwarfstring_append_printf_u(&s," is too high. "
             "Valid units must be < %u ",xuhdr->gx_units_in_index);
-        _dwarf_error_string(dbg, err, DW_DLE_XU_NAME_COL_ERROR,
+        _dwarf_error_string(dbg, error, DW_DLE_XU_NAME_COL_ERROR,
             dwarfstring_string(&s));
         dwarfstring_destructor(&s);
         return DW_DLV_ERROR;
     }
 
-    if (column_index >=  xuhdr->gx_column_count_sections) {
+    if (column_index >= column_count) {
         dwarfstring s;
 
         dwarfstring_constructor(&s);
@@ -611,8 +801,8 @@ dwarf_get_xu_section_offset(Dwarf_Xu_Index_Header xuhdr,
             "column index of %u ",column_index);
         dwarfstring_append_printf_u(&s," is too high. "
             "Valid column indexes  must be < %u ",
-            xuhdr->gx_column_count_sections);
-        _dwarf_error_string(dbg, err, DW_DLE_XU_NAME_COL_ERROR,
+            column_count);
+        _dwarf_error_string(dbg, error, DW_DLE_XU_NAME_COL_ERROR,
             dwarfstring_string(&s));
         dwarfstring_destructor(&s);
         return DW_DLV_ERROR;
@@ -620,43 +810,128 @@ dwarf_get_xu_section_offset(Dwarf_Xu_Index_Header xuhdr,
     /*  As noted above we have hidden the extra initial
         row from the offsets table so it is just
         0 to U-1. */
-    offsetrow = offsetrow + (row_index*column_count * SIZEOFT32);
-    offsetentry = offsetrow + (column_index *  SIZEOFT32);
-
-    sizerow = sizerow + (row_index*column_count * SIZEOFT32);
-    sizeentry = sizerow + (column_index *  SIZEOFT32);
+    ofrow = row_index*column_count * SIZEOFT32;
+    ofentry = column_index *  SIZEOFT32;
+    offsetrow = offsetrow + ofrow;
+    offsetentry = offsetrow + ofentry;
+    if (ofrow >= secsize) {
+        _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+            "DW_DLE_XU_NAME_COL_ERROR row offset past end of"
+            " section.");
+        return DW_DLV_ERROR;
+    }
+    if (ofentry >= secsize) {
+        _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+            "DW_DLE_XU_NAME_COL_ERROR entry offset past end of"
+            " section.");
+        return DW_DLV_ERROR;
+    }
 
+    sirow =  row_index*column_count * SIZEOFT32;
+    sientry = column_index *  SIZEOFT32;
+    sizerow = sizerow + sirow;
+    sizeentry = sizerow + sientry;
+    if (sirow > secsize) {
+        _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+            "DW_DLE_XU_NAME_COL_ERROR row size past end of"
+            " section.");
+        return DW_DLV_ERROR;
+    }
+    if (sientry > secsize) {
+        _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+            "DW_DLE_XU_NAME_COL_ERROR entry size larger than"
+            " section.");
+        return DW_DLV_ERROR;
+    }
     {
         READ_UNALIGNED_CK(dbg,offset,Dwarf_Unsigned,
             offsetentry,
-            SIZEOFT32,err,section_end);
+            SIZEOFT32,error,section_end);
         offsetentry += SIZEOFT32;
 
         READ_UNALIGNED_CK(dbg,size,Dwarf_Unsigned,
             sizeentry,
-            SIZEOFT32,err,section_end);
+            SIZEOFT32,error,section_end);
         sizeentry += SIZEOFT32;
     }
+    /*  These are offset-in and size-of a section which
+        is separate from .debug_cu/tu_index sections.
+        _cu_/_tu_ */
+    if (!size) {
+        /*  If zero, this entry makes no contribution
+            to a section in the package file.
+            DWARF5 Sec 7.3.5.3   page 194. */
+        *sec_offset = offset;
+        *sec_size =  size;
+        return DW_DLV_OK;
+    }
+    res = _dwarf_find_xu_target_sec_size(dbg, xuhdr,
+        column_index,
+        &target_sec_size,error);
+    if (res == DW_DLV_ERROR) {
+        return res;
+    }
+    if (res == DW_DLV_NO_ENTRY) {
+        /*  These are not precise tests.  */
+        if (offset > dbg->de_filesize) {
+            _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+                "DW_DLE_XU_NAME_COL_ERROR area offset larger than"
+                " file size.");
+            return DW_DLV_ERROR;
+        }
+        if (size > dbg->de_filesize) {
+            _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+                "DW_DLE_XU_NAME_COL_ERROR area size larger than"
+                " file size.");
+            return DW_DLV_ERROR;
+        }
+    } else {
+        if (offset && (offset >= target_sec_size)) {
+            dwarfstring m;
+
+            dwarfstring_constructor(&m);
+            dwarfstring_append_printf_u(&m,
+                "DW_DLE_XU_NAME_COL_ERROR area offset 0x%x ",offset);
+            dwarfstring_append_printf_u(&m," larger than"
+                " target section size of 0x%x.",target_sec_size);
+            _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+                dwarfstring_string(&m));
+            dwarfstring_destructor(&m);
+            return DW_DLV_ERROR;
+        }
+        if (size > target_sec_size) {
+            dwarfstring m;
+            dwarfstring_constructor(&m);
+
+            dwarfstring_append_printf_u(&m,
+                "DW_DLE_XU_NAME_COL_ERROR area size 0x%x ",size);
+            dwarfstring_append_printf_u(&m," larger than"
+                " target section size of 0x%x.",target_sec_size);
+            _dwarf_error_string(dbg,error,DW_DLE_XU_NAME_COL_ERROR,
+                dwarfstring_string(&m));
+            dwarfstring_destructor(&m);
+            return DW_DLV_ERROR;
+        }
+    }
     *sec_offset = offset;
     *sec_size =  size;
     return DW_DLV_OK;
 }
 
 static int
-_dwarf_search_fission_for_key(UNUSEDARG Dwarf_Debug dbg,
+_dwarf_search_fission_for_key(Dwarf_Debug dbg,
     Dwarf_Xu_Index_Header xuhdr,
     Dwarf_Sig8 *key_in,
     Dwarf_Unsigned * percu_index_out,
     Dwarf_Error *error)
 {
     Dwarf_Unsigned key = 0;
-    Dwarf_Unsigned primary_hash = 0;
-    Dwarf_Unsigned hashprime = 0;
     Dwarf_Unsigned slots =  xuhdr->gx_slots_in_hash;
-    Dwarf_Unsigned mask = slots -1;
     Dwarf_Sig8 hashentry_key;
     Dwarf_Unsigned percu_index = 0;
+    Dwarf_Unsigned h = 0;
 
+    (void)dbg;
     hashentry_key = zerohashkey;
     /*  Look for corrupt section data. */
     if (slots > xuhdr->gx_section_length) {
@@ -674,6 +949,9 @@ _dwarf_search_fission_for_key(UNUSEDARG Dwarf_Debug dbg,
         dwarfstring_destructor(&s);
         return DW_DLV_ERROR;
     }
+    if (!slots) {
+        return DW_DLV_NO_ENTRY;
+    }
     if ( (4*slots) > xuhdr->gx_section_length) {
         dwarfstring s;
 
@@ -689,34 +967,42 @@ _dwarf_search_fission_for_key(UNUSEDARG Dwarf_Debug dbg,
         dwarfstring_destructor(&s);
         return DW_DLV_ERROR;
     }
-    if (sizeof (key) != sizeof(key_in)) {
+    if (sizeof(key) != sizeof(*key_in)) {
         /* The hash won't work right in this case */
-        _dwarf_error(dbg, error, DW_DLE_XU_HASH_ROW_ERROR);
+        dwarfstring s;
+
+        dwarfstring_constructor(&s);
+        dwarfstring_append_printf_u(&s,
+            "DW_DLE_XU_HASH_ROW_ERROR "
+            "We require 8 byte keys, not %u byte keys",
+            sizeof(key));
+        _dwarf_error_string(dbg, error, DW_DLE_XU_HASH_ROW_ERROR,
+            dwarfstring_string(&s));
+        dwarfstring_destructor(&s);
+        return DW_DLV_ERROR;
     }
     ASNARL(key,key_in,sizeof(*key_in));
-    primary_hash = key & mask;
-    hashprime =  (((key >>32) &mask) |1);
-    while (1) {
+    /*  If we built a lookup based on hash key
+        this loop would not be necessary. */
+    for (h = 0; h < slots; ++h) {
         int res = 0;
 
         res = dwarf_get_xu_hash_entry(xuhdr,
-            primary_hash,&hashentry_key,
+            h,&hashentry_key,
             &percu_index,error);
         if (res != DW_DLV_OK) {
             return res;
         }
         if (percu_index == 0 &&
             !memcmp(&hashentry_key,&zerohashkey,sizeof(Dwarf_Sig8))) {
-            return DW_DLV_NO_ENTRY;
+            continue;
         }
         if (!memcmp(key_in,&hashentry_key,sizeof(Dwarf_Sig8))) {
             /* FOUND */
             *percu_index_out = percu_index;
             return  DW_DLV_OK;
         }
-        primary_hash = (primary_hash + hashprime) % slots;
     }
-    /* ASSERT: Cannot get here. */
     return DW_DLV_NO_ENTRY;
 }
 
@@ -736,7 +1022,8 @@ _dwarf_search_fission_for_offset(Dwarf_Debug dbg,
 {
     Dwarf_Unsigned i = 0;
     Dwarf_Unsigned m = 0;
-    int secnum_index = -1;  /* N index */
+    Dwarf_Unsigned secnum_index = 0;
+    Dwarf_Bool     found_secnum = FALSE;
     int res = 0;
 
     for ( i = 0; i< xuhdr->gx_column_count_sections; i++) {
@@ -750,10 +1037,11 @@ _dwarf_search_fission_for_offset(Dwarf_Debug dbg,
         }
         if (num == dfp_sect_num) {
             secnum_index = i;
+            found_secnum = TRUE;
             break;
         }
     }
-    if (secnum_index == -1) {
+    if (FALSE == found_secnum) {
         _dwarf_error(dbg,error,DW_DLE_FISSION_SECNUM_ERR);
         return DW_DLV_ERROR;
     }
@@ -820,12 +1108,14 @@ transform_xu_to_dfp(Dwarf_Xu_Index_Header xuhdr,
     Dwarf_Debug_Fission_Per_CU *  percu_out,
     Dwarf_Error *error)
 {
-    unsigned i = 0;
-    unsigned l = 0;
-    unsigned n = 1;
-    unsigned max_cols = xuhdr->gx_column_count_sections;  /* L */
-    unsigned secnums[DW_FISSION_SECT_COUNT];
-    int res;
+    Dwarf_Unsigned i = 0;
+    Dwarf_Unsigned l = 0;
+    Dwarf_Unsigned n = 1;
+    Dwarf_Unsigned max_cols =
+        xuhdr->gx_column_count_sections;/* L */
+    Dwarf_Unsigned secnums[DW_FISSION_SECT_COUNT];
+    int res = 0;
+
     for ( i = 0; i< max_cols; i++) {
         /*  We could put the secnums array into xuhdr
             if recreating it is too slow. */
@@ -841,7 +1131,7 @@ transform_xu_to_dfp(Dwarf_Xu_Index_Header xuhdr,
     for (l = 0; l < max_cols; ++l) {  /* L */
         Dwarf_Unsigned sec_off = 0;
         Dwarf_Unsigned sec_size = 0;
-        unsigned l_as_sect = secnums[l];
+        Dwarf_Unsigned l_as_sect = secnums[l];
         res = dwarf_get_xu_section_offset(xuhdr,n,l,
             &sec_off,&sec_size,error);
         if (res != DW_DLV_OK) {
@@ -872,6 +1162,7 @@ _dwarf_get_debugfission_for_offset(Dwarf_Debug dbg,
     Dwarf_Unsigned sect_index_base = 0;
     Dwarf_Sig8 key;
 
+    CHECK_DBG(dbg,error,"_dwarf_get_debugfission_for_offset()");
     sect_index_base = DW_SECT_INFO;
     key = zerohashkey;
     sres = _dwarf_get_xuhdr(dbg,key_type, &xuhdr,error);
@@ -892,15 +1183,24 @@ _dwarf_get_debugfission_for_offset(Dwarf_Debug dbg,
 }
 int
 dwarf_get_debugfission_for_key(Dwarf_Debug dbg,
-    Dwarf_Sig8 *  key  /* pointer to hash signature */,
-    const char * key_type  /*  "cu" or "tu" */,
+    Dwarf_Sig8  *key  /* pointer to hash signature */,
+    const char  *key_type  /*  "cu" or "tu" */,
     Dwarf_Debug_Fission_Per_CU *  percu_out,
-    Dwarf_Error *  error )
+    Dwarf_Error *error)
 {
     int sres = 0;
     Dwarf_Unsigned percu_index = 0;
     Dwarf_Xu_Index_Header xuhdr = 0;
 
+    CHECK_DBG(dbg,error,"dwarf_get_debugfission_for_key()");
+    if (!key || !key_type || !percu_out) {
+        _dwarf_error_string(0,error,DW_DLE_XU_TYPE_ARG_ERROR,
+            "DW_DLE_XU_TYPE_ARG_ERROR: dw_key, dw_keytype, or "
+            "Dwarf_Debug_Fission_Per_CU pointer* to return "
+            "is not valid");
+        return DW_DLV_ERROR;
+    }
+
     sres = _dwarf_load_debug_info(dbg,error);
     if (sres == DW_DLV_ERROR) {
         return sres;
@@ -926,7 +1226,7 @@ dwarf_get_debugfission_for_key(Dwarf_Debug dbg,
 }
 
 void
-dwarf_xu_header_free(Dwarf_Xu_Index_Header indexptr)
+dwarf_dealloc_xu_header(Dwarf_Xu_Index_Header indexptr)
 {
     if (indexptr) {
         Dwarf_Debug dbg = indexptr->gx_dbg;
